"""Tests for the Ammunition and AsyncAmmunition resource classes."""

from __future__ import annotations

from urllib.parse import quote

from gunspec._resources._ammunition import Ammunition, AsyncAmmunition


class TestAmmunition:
    """Sync Ammunition resource tests."""

    def test_list(self, mock_sync_client):
        resource = Ammunition(mock_sync_client)
        params = {"page": 1, "limit": 10}
        resource.list(params)
        mock_sync_client.get_paginated.assert_called_once_with(
            "/v1/ammunition", query=params
        )

    def test_list_no_params(self, mock_sync_client):
        resource = Ammunition(mock_sync_client)
        resource.list()
        mock_sync_client.get_paginated.assert_called_once_with(
            "/v1/ammunition", query=None
        )

    def test_get(self, mock_sync_client):
        resource = Ammunition(mock_sync_client)
        resource.get("federal-hst-9mm")
        mock_sync_client.get.assert_called_once_with(
            f"/v1/ammunition/{quote('federal-hst-9mm', safe='')}"
        )

    def test_get_bullet_svg(self, mock_sync_client):
        resource = Ammunition(mock_sync_client)
        resource.get_bullet_svg("federal-hst-9mm")
        mock_sync_client.get.assert_called_once_with(
            f"/v1/ammunition/{quote('federal-hst-9mm', safe='')}/bullet.svg"
        )

    def test_ballistics(self, mock_sync_client):
        resource = Ammunition(mock_sync_client)
        params = {"range": 100}
        resource.ballistics("federal-hst-9mm", params)
        mock_sync_client.get.assert_called_once_with(
            f"/v1/ammunition/{quote('federal-hst-9mm', safe='')}/ballistics",
            query=params,
        )

    def test_ballistics_no_params(self, mock_sync_client):
        resource = Ammunition(mock_sync_client)
        resource.ballistics("federal-hst-9mm")
        mock_sync_client.get.assert_called_once_with(
            f"/v1/ammunition/{quote('federal-hst-9mm', safe='')}/ballistics",
            query=None,
        )


class TestAsyncAmmunition:
    """Async Ammunition resource tests."""

    async def test_list(self, mock_async_client):
        resource = AsyncAmmunition(mock_async_client)
        params = {"page": 1, "limit": 10}
        await resource.list(params)
        mock_async_client.get_paginated.assert_called_once_with(
            "/v1/ammunition", query=params
        )

    async def test_list_no_params(self, mock_async_client):
        resource = AsyncAmmunition(mock_async_client)
        await resource.list()
        mock_async_client.get_paginated.assert_called_once_with(
            "/v1/ammunition", query=None
        )

    async def test_get(self, mock_async_client):
        resource = AsyncAmmunition(mock_async_client)
        await resource.get("federal-hst-9mm")
        mock_async_client.get.assert_called_once_with(
            f"/v1/ammunition/{quote('federal-hst-9mm', safe='')}"
        )

    async def test_get_bullet_svg(self, mock_async_client):
        resource = AsyncAmmunition(mock_async_client)
        await resource.get_bullet_svg("federal-hst-9mm")
        mock_async_client.get.assert_called_once_with(
            f"/v1/ammunition/{quote('federal-hst-9mm', safe='')}/bullet.svg"
        )

    async def test_ballistics(self, mock_async_client):
        resource = AsyncAmmunition(mock_async_client)
        params = {"range": 100}
        await resource.ballistics("federal-hst-9mm", params)
        mock_async_client.get.assert_called_once_with(
            f"/v1/ammunition/{quote('federal-hst-9mm', safe='')}/ballistics",
            query=params,
        )

    async def test_ballistics_no_params(self, mock_async_client):
        resource = AsyncAmmunition(mock_async_client)
        await resource.ballistics("federal-hst-9mm")
        mock_async_client.get.assert_called_once_with(
            f"/v1/ammunition/{quote('federal-hst-9mm', safe='')}/ballistics",
            query=None,
        )



"""Tests for the Usage and AsyncUsage resource classes."""

from __future__ import annotations

from gunspec._resources._usage import AsyncUsage, Usage


class TestUsage:
    """Sync Usage resource tests."""

    def test_get(self, mock_sync_client):
        resource = Usage(mock_sync_client)
        params = {"period": "monthly"}
        resource.get(params)
        mock_sync_client.get.assert_called_once_with("/v1/me/usage", query=params)

    def test_get_no_params(self, mock_sync_client):
        resource = Usage(mock_sync_client)
        resource.get()
        mock_sync_client.get.assert_called_once_with("/v1/me/usage", query=None)


class TestAsyncUsage:
    """Async Usage resource tests."""

    async def test_get(self, mock_async_client):
        resource = AsyncUsage(mock_async_client)
        params = {"period": "monthly"}
        await resource.get(params)
        mock_async_client.get.assert_called_once_with("/v1/me/usage", query=params)

    async def test_get_no_params(self, mock_async_client):
        resource = AsyncUsage(mock_async_client)
        await resource.get()
        mock_async_client.get.assert_called_once_with("/v1/me/usage", query=None)



from __future__ import annotations

from typing import TYPE_CHECKING, Dict

if TYPE_CHECKING:
    from .._core._http_client import APIResponse, AsyncHttpClient, SyncHttpClient


class Usage:
    """Sync usage resource."""

    def __init__(self, client: SyncHttpClient) -> None:
        self._client = client

    def get(self, params: Dict | None = None) -> APIResponse:
        return self._client.get("/v1/me/usage", query=params)


class AsyncUsage:
    """Async usage resource."""

    def __init__(self, client: AsyncHttpClient) -> None:
        self._client = client

    async def get(self, params: Dict | None = None) -> APIResponse:
        return await self._client.get("/v1/me/usage", query=params)



from __future__ import annotations

from typing import TypedDict


class ErrorDetail(TypedDict):
    """Individual error detail returned by the API."""

    code: str
    message: str


class APIErrorResponse(TypedDict):
    """Top-level error response envelope."""

    success: bool
    error: ErrorDetail



"""Tests for the Firearms and AsyncFirearms resource classes."""

from __future__ import annotations

from urllib.parse import quote

from gunspec._resources._firearms import AsyncFirearms, Firearms


class TestFirearms:
    """Sync Firearms resource tests."""

    def test_list(self, mock_sync_client):
        resource = Firearms(mock_sync_client)
        params = {"page": 1, "limit": 10}
        resource.list(params)
        mock_sync_client.get_paginated.assert_called_once_with(
            "/v1/firearms", query=params
        )

    def test_list_no_params(self, mock_sync_client):
        resource = Firearms(mock_sync_client)
        resource.list()
        mock_sync_client.get_paginated.assert_called_once_with(
            "/v1/firearms", query=None
        )

    def test_search(self, mock_sync_client):
        resource = Firearms(mock_sync_client)
        params = {"q": "glock"}
        resource.search(params)
        mock_sync_client.get_paginated.assert_called_once_with(
            "/v1/firearms/search", query=params
        )

    def test_compare(self, mock_sync_client):
        resource = Firearms(mock_sync_client)
        params = {"ids": "glock-g17,sig-p320"}
        resource.compare(params)
        mock_sync_client.get.assert_called_once_with(
            "/v1/firearms/compare", query=params
        )

    def test_game_meta(self, mock_sync_client):
        resource = Firearms(mock_sync_client)
        params = {"category": "pistol"}
        resource.game_meta(params)
        mock_sync_client.get.assert_called_once_with(
            "/v1/firearms/game-meta", query=params
        )

    def test_action_types(self, mock_sync_client):
        resource = Firearms(mock_sync_client)
        resource.action_types()
        mock_sync_client.get.assert_called_once_with("/v1/firearms/action-types")

    def test_filter_options(self, mock_sync_client):
        resource = Firearms(mock_sync_client)
        resource.filter_options()
        mock_sync_client.get.assert_called_once_with("/v1/firearms/filter-options")

    def test_random(self, mock_sync_client):
        resource = Firearms(mock_sync_client)
        params = {"count": 5}
        resource.random(params)
        mock_sync_client.get.assert_called_once_with(
            "/v1/firearms/random", query=params
        )

    def test_top(self, mock_sync_client):
        resource = Firearms(mock_sync_client)
        params = {"metric": "popularity", "limit": 10}
        resource.top(params)
        mock_sync_client.get.assert_called_once_with("/v1/firearms/top", query=params)

    def test_head_to_head(self, mock_sync_client):
        resource = Firearms(mock_sync_client)
        params = {"a": "glock-g17", "b": "sig-p320"}
        resource.head_to_head(params)
        mock_sync_client.get.assert_called_once_with(
            "/v1/firearms/head-to-head", query=params
        )

    def test_by_feature(self, mock_sync_client):
        resource = Firearms(mock_sync_client)
        params = {"feature": "rail"}
        resource.by_feature(params)
        mock_sync_client.get_paginated.assert_called_once_with(
            "/v1/firearms/by-feature", query=params
        )

    def test_by_action(self, mock_sync_client):
        resource = Firearms(mock_sync_client)
        params = {"action": "semi-automatic"}
        resource.by_action(params)
        mock_sync_client.get_paginated.assert_called_once_with(
            "/v1/firearms/by-action", query=params
        )

    def test_by_material(self, mock_sync_client):
        resource = Firearms(mock_sync_client)
        params = {"material": "polymer"}
        resource.by_material(params)
        mock_sync_client.get_paginated.assert_called_once_with(
            "/v1/firearms/by-material", query=params
        )

    def test_by_designer(self, mock_sync_client):
        resource = Firearms(mock_sync_client)
        params = {"designer": "John Browning"}
        resource.by_designer(params)
        mock_sync_client.get_paginated.assert_called_once_with(
            "/v1/firearms/by-designer", query=params
        )

    def test_power_rating(self, mock_sync_client):
        resource = Firearms(mock_sync_client)
        params = {"min": 5}
        resource.power_rating(params)
        mock_sync_client.get_paginated.assert_called_once_with(
            "/v1/firearms/power-rating", query=params
        )

    def test_timeline(self, mock_sync_client):
        resource = Firearms(mock_sync_client)
        params = {"start": 1900, "end": 2000}
        resource.timeline(params)
        mock_sync_client.get_paginated.assert_called_once_with(
            "/v1/firearms/timeline", query=params
        )

    def test_by_conflict(self, mock_sync_client):
        resource = Firearms(mock_sync_client)
        params = {"conflict": "ww2"}
        resource.by_conflict(params)
        mock_sync_client.get_paginated.assert_called_once_with(
            "/v1/firearms/by-conflict", query=params
        )

    def test_get(self, mock_sync_client):
        resource = Firearms(mock_sync_client)
        resource.get("glock-g17")
        mock_sync_client.get.assert_called_once_with(
            f"/v1/firearms/{quote('glock-g17', safe='')}"
        )

    def test_get_variants(self, mock_sync_client):
        resource = Firearms(mock_sync_client)
        resource.get_variants("glock-g17")
        mock_sync_client.get.assert_called_once_with(
            f"/v1/firearms/{quote('glock-g17', safe='')}/variants"
        )

    def test_get_images(self, mock_sync_client):
        resource = Firearms(mock_sync_client)
        resource.get_images("glock-g17")
        mock_sync_client.get.assert_called_once_with(
            f"/v1/firearms/{quote('glock-g17', safe='')}/images"
        )

    def test_get_game_stats(self, mock_sync_client):
        resource = Firearms(mock_sync_client)
        resource.get_game_stats("glock-g17")
        mock_sync_client.get.assert_called_once_with(
            f"/v1/firearms/{quote('glock-g17', safe='')}/game-stats"
        )

    def test_get_dimensions(self, mock_sync_client):
        resource = Firearms(mock_sync_client)
        resource.get_dimensions("glock-g17")
        mock_sync_client.get.assert_called_once_with(
            f"/v1/firearms/{quote('glock-g17', safe='')}/dimensions"
        )

    def test_get_users(self, mock_sync_client):
        resource = Firearms(mock_sync_client)
        resource.get_users("glock-g17")
        mock_sync_client.get.assert_called_once_with(
            f"/v1/firearms/{quote('glock-g17', safe='')}/users"
        )

    def test_get_family_tree(self, mock_sync_client):
        resource = Firearms(mock_sync_client)
        resource.get_family_tree("glock-g17")
        mock_sync_client.get.assert_called_once_with(
            f"/v1/firearms/{quote('glock-g17', safe='')}/family-tree"
        )

    def test_get_similar(self, mock_sync_client):
        resource = Firearms(mock_sync_client)
        resource.get_similar("glock-g17")
        mock_sync_client.get.assert_called_once_with(
            f"/v1/firearms/{quote('glock-g17', safe='')}/similar"
        )

    def test_get_adoption_map(self, mock_sync_client):
        resource = Firearms(mock_sync_client)
        resource.get_adoption_map("glock-g17")
        mock_sync_client.get.assert_called_once_with(
            f"/v1/firearms/{quote('glock-g17', safe='')}/adoption-map"
        )

    def test_get_game_profile(self, mock_sync_client):
        resource = Firearms(mock_sync_client)
        resource.get_game_profile("glock-g17")
        mock_sync_client.get.assert_called_once_with(
            f"/v1/firearms/{quote('glock-g17', safe='')}/game-profile"
        )

    def test_get_silhouette(self, mock_sync_client):
        resource = Firearms(mock_sync_client)
        params = {"format": "svg"}
        resource.get_silhouette("glock-g17", params)
        mock_sync_client.get.assert_called_once_with(
            f"/v1/firearms/{quote('glock-g17', safe='')}/silhouette", query=params
        )

    def test_get_silhouette_no_params(self, mock_sync_client):
        resource = Firearms(mock_sync_client)
        resource.get_silhouette("glock-g17")
        mock_sync_client.get.assert_called_once_with(
            f"/v1/firearms/{quote('glock-g17', safe='')}/silhouette", query=None
        )

    def test_calculate(self, mock_sync_client):
        resource = Firearms(mock_sync_client)
        params = {"range": 100}
        resource.calculate("glock-g17", params)
        mock_sync_client.get.assert_called_once_with(
            f"/v1/firearms/{quote('glock-g17', safe='')}/calculate", query=params
        )

    def test_load(self, mock_sync_client):
        resource = Firearms(mock_sync_client)
        params = {"type": "full"}
        resource.load("glock-g17", params)
        mock_sync_client.get.assert_called_once_with(
            f"/v1/firearms/{quote('glock-g17', safe='')}/load", query=params
        )


class TestAsyncFirearms:
    """Async Firearms resource tests."""

    async def test_list(self, mock_async_client):
        resource = AsyncFirearms(mock_async_client)
        params = {"page": 1, "limit": 10}
        await resource.list(params)
        mock_async_client.get_paginated.assert_called_once_with(
            "/v1/firearms", query=params
        )

    async def test_list_no_params(self, mock_async_client):
        resource = AsyncFirearms(mock_async_client)
        await resource.list()
        mock_async_client.get_paginated.assert_called_once_with(
            "/v1/firearms", query=None
        )

    async def test_search(self, mock_async_client):
        resource = AsyncFirearms(mock_async_client)
        params = {"q": "glock"}
        await resource.search(params)
        mock_async_client.get_paginated.assert_called_once_with(
            "/v1/firearms/search", query=params
        )

    async def test_compare(self, mock_async_client):
        resource = AsyncFirearms(mock_async_client)
        params = {"ids": "glock-g17,sig-p320"}
        await resource.compare(params)
        mock_async_client.get.assert_called_once_with(
            "/v1/firearms/compare", query=params
        )

    async def test_game_meta(self, mock_async_client):
        resource = AsyncFirearms(mock_async_client)
        params = {"category": "pistol"}
        await resource.game_meta(params)
        mock_async_client.get.assert_called_once_with(
            "/v1/firearms/game-meta", query=params
        )

    async def test_action_types(self, mock_async_client):
        resource = AsyncFirearms(mock_async_client)
        await resource.action_types()
        mock_async_client.get.assert_called_once_with("/v1/firearms/action-types")

    async def test_filter_options(self, mock_async_client):
        resource = AsyncFirearms(mock_async_client)
        await resource.filter_options()
        mock_async_client.get.assert_called_once_with("/v1/firearms/filter-options")

    async def test_random(self, mock_async_client):
        resource = AsyncFirearms(mock_async_client)
        params = {"count": 5}
        await resource.random(params)
        mock_async_client.get.assert_called_once_with(
            "/v1/firearms/random", query=params
        )

    async def test_top(self, mock_async_client):
        resource = AsyncFirearms(mock_async_client)
        params = {"metric": "popularity", "limit": 10}
        await resource.top(params)
        mock_async_client.get.assert_called_once_with(
            "/v1/firearms/top", query=params
        )

    async def test_head_to_head(self, mock_async_client):
        resource = AsyncFirearms(mock_async_client)
        params = {"a": "glock-g17", "b": "sig-p320"}
        await resource.head_to_head(params)
        mock_async_client.get.assert_called_once_with(
            "/v1/firearms/head-to-head", query=params
        )

    async def test_by_feature(self, mock_async_client):
        resource = AsyncFirearms(mock_async_client)
        params = {"feature": "rail"}
        await resource.by_feature(params)
        mock_async_client.get_paginated.assert_called_once_with(
            "/v1/firearms/by-feature", query=params
        )

    async def test_by_action(self, mock_async_client):
        resource = AsyncFirearms(mock_async_client)
        params = {"action": "semi-automatic"}
        await resource.by_action(params)
        mock_async_client.get_paginated.assert_called_once_with(
            "/v1/firearms/by-action", query=params
        )

    async def test_by_material(self, mock_async_client):
        resource = AsyncFirearms(mock_async_client)
        params = {"material": "polymer"}
        await resource.by_material(params)
        mock_async_client.get_paginated.assert_called_once_with(
            "/v1/firearms/by-material", query=params
        )

    async def test_by_designer(self, mock_async_client):
        resource = AsyncFirearms(mock_async_client)
        params = {"designer": "John Browning"}
        await resource.by_designer(params)
        mock_async_client.get_paginated.assert_called_once_with(
            "/v1/firearms/by-designer", query=params
        )

    async def test_power_rating(self, mock_async_client):
        resource = AsyncFirearms(mock_async_client)
        params = {"min": 5}
        await resource.power_rating(params)
        mock_async_client.get_paginated.assert_called_once_with(
            "/v1/firearms/power-rating", query=params
        )

    async def test_timeline(self, mock_async_client):
        resource = AsyncFirearms(mock_async_client)
        params = {"start": 1900, "end": 2000}
        await resource.timeline(params)
        mock_async_client.get_paginated.assert_called_once_with(
            "/v1/firearms/timeline", query=params
        )

    async def test_by_conflict(self, mock_async_client):
        resource = AsyncFirearms(mock_async_client)
        params = {"conflict": "ww2"}
        await resource.by_conflict(params)
        mock_async_client.get_paginated.assert_called_once_with(
            "/v1/firearms/by-conflict", query=params
        )

    async def test_get(self, mock_async_client):
        resource = AsyncFirearms(mock_async_client)
        await resource.get("glock-g17")
        mock_async_client.get.assert_called_once_with(
            f"/v1/firearms/{quote('glock-g17', safe='')}"
        )

    async def test_get_variants(self, mock_async_client):
        resource = AsyncFirearms(mock_async_client)
        await resource.get_variants("glock-g17")
        mock_async_client.get.assert_called_once_with(
            f"/v1/firearms/{quote('glock-g17', safe='')}/variants"
        )

    async def test_get_images(self, mock_async_client):
        resource = AsyncFirearms(mock_async_client)
        await resource.get_images("glock-g17")
        mock_async_client.get.assert_called_once_with(
            f"/v1/firearms/{quote('glock-g17', safe='')}/images"
        )

    async def test_get_game_stats(self, mock_async_client):
        resource = AsyncFirearms(mock_async_client)
        await resource.get_game_stats("glock-g17")
        mock_async_client.get.assert_called_once_with(
            f"/v1/firearms/{quote('glock-g17', safe='')}/game-stats"
        )

    async def test_get_dimensions(self, mock_async_client):
        resource = AsyncFirearms(mock_async_client)
        await resource.get_dimensions("glock-g17")
        mock_async_client.get.assert_called_once_with(
            f"/v1/firearms/{quote('glock-g17', safe='')}/dimensions"
        )

    async def test_get_users(self, mock_async_client):
        resource = AsyncFirearms(mock_async_client)
        await resource.get_users("glock-g17")
        mock_async_client.get.assert_called_once_with(
            f"/v1/firearms/{quote('glock-g17', safe='')}/users"
        )

    async def test_get_family_tree(self, mock_async_client):
        resource = AsyncFirearms(mock_async_client)
        await resource.get_family_tree("glock-g17")
        mock_async_client.get.assert_called_once_with(
            f"/v1/firearms/{quote('glock-g17', safe='')}/family-tree"
        )

    async def test_get_similar(self, mock_async_client):
        resource = AsyncFirearms(mock_async_client)
        await resource.get_similar("glock-g17")
        mock_async_client.get.assert_called_once_with(
            f"/v1/firearms/{quote('glock-g17', safe='')}/similar"
        )

    async def test_get_adoption_map(self, mock_async_client):
        resource = AsyncFirearms(mock_async_client)
        await resource.get_adoption_map("glock-g17")
        mock_async_client.get.assert_called_once_with(
            f"/v1/firearms/{quote('glock-g17', safe='')}/adoption-map"
        )

    async def test_get_game_profile(self, mock_async_client):
        resource = AsyncFirearms(mock_async_client)
        await resource.get_game_profile("glock-g17")
        mock_async_client.get.assert_called_once_with(
            f"/v1/firearms/{quote('glock-g17', safe='')}/game-profile"
        )

    async def test_get_silhouette(self, mock_async_client):
        resource = AsyncFirearms(mock_async_client)
        params = {"format": "svg"}
        await resource.get_silhouette("glock-g17", params)
        mock_async_client.get.assert_called_once_with(
            f"/v1/firearms/{quote('glock-g17', safe='')}/silhouette", query=params
        )

    async def test_get_silhouette_no_params(self, mock_async_client):
        resource = AsyncFirearms(mock_async_client)
        await resource.get_silhouette("glock-g17")
        mock_async_client.get.assert_called_once_with(
            f"/v1/firearms/{quote('glock-g17', safe='')}/silhouette", query=None
        )

    async def test_calculate(self, mock_async_client):
        resource = AsyncFirearms(mock_async_client)
        params = {"range": 100}
        await resource.calculate("glock-g17", params)
        mock_async_client.get.assert_called_once_with(
            f"/v1/firearms/{quote('glock-g17', safe='')}/calculate", query=params
        )

    async def test_load(self, mock_async_client):
        resource = AsyncFirearms(mock_async_client)
        params = {"type": "full"}
        await resource.load("glock-g17", params)
        mock_async_client.get.assert_called_once_with(
            f"/v1/firearms/{quote('glock-g17', safe='')}/load", query=params
        )



from __future__ import annotations

from typing import TYPE_CHECKING, Dict

if TYPE_CHECKING:
    from .._core._http_client import APIResponse, AsyncHttpClient, PaginatedResponse, SyncHttpClient


class DataQuality:
    """Sync data quality resource."""

    def __init__(self, client: SyncHttpClient) -> None:
        self._client = client

    def coverage(self) -> APIResponse:
        return self._client.get("/v1/data/coverage")

    def confidence(self, params: Dict | None = None) -> PaginatedResponse:
        return self._client.get_paginated("/v1/data/confidence", query=params)


class AsyncDataQuality:
    """Async data quality resource."""

    def __init__(self, client: AsyncHttpClient) -> None:
        self._client = client

    async def coverage(self) -> APIResponse:
        return await self._client.get("/v1/data/coverage")

    async def confidence(self, params: Dict | None = None) -> PaginatedResponse:
        return await self._client.get_paginated("/v1/data/confidence", query=params)



from __future__ import annotations

from typing import TYPE_CHECKING, Dict

if TYPE_CHECKING:
    from .._core._http_client import APIResponse, AsyncHttpClient, PaginatedResponse, SyncHttpClient


class Reports:
    """Sync reports resource."""

    def __init__(self, client: SyncHttpClient) -> None:
        self._client = client

    def create(self, params: Dict) -> APIResponse:
        return self._client.post("/v1/me/reports", body=params)

    def list(self, params: Dict | None = None) -> PaginatedResponse:
        return self._client.get_paginated("/v1/me/reports", query=params)


class AsyncReports:
    """Async reports resource."""

    def __init__(self, client: AsyncHttpClient) -> None:
        self._client = client

    async def create(self, params: Dict) -> APIResponse:
        return await self._client.post("/v1/me/reports", body=params)

    async def list(self, params: Dict | None = None) -> PaginatedResponse:
        return await self._client.get_paginated("/v1/me/reports", query=params)



"""Tests for the Favorites and AsyncFavorites resource classes."""

from __future__ import annotations

from urllib.parse import quote

from gunspec._resources._favorites import AsyncFavorites, Favorites


class TestFavorites:
    """Sync Favorites resource tests."""

    def test_list(self, mock_sync_client):
        resource = Favorites(mock_sync_client)
        params = {"page": 1, "limit": 10}
        resource.list(params)
        mock_sync_client.get_paginated.assert_called_once_with(
            "/v1/me/favorites", query=params
        )

    def test_list_no_params(self, mock_sync_client):
        resource = Favorites(mock_sync_client)
        resource.list()
        mock_sync_client.get_paginated.assert_called_once_with(
            "/v1/me/favorites", query=None
        )

    def test_add(self, mock_sync_client):
        resource = Favorites(mock_sync_client)
        resource.add("glock-g17")
        mock_sync_client.post.assert_called_once_with(
            f"/v1/me/favorites/{quote('glock-g17', safe='')}"
        )

    def test_remove(self, mock_sync_client):
        resource = Favorites(mock_sync_client)
        resource.remove("glock-g17")
        mock_sync_client.delete.assert_called_once_with(
            f"/v1/me/favorites/{quote('glock-g17', safe='')}"
        )


class TestAsyncFavorites:
    """Async Favorites resource tests."""

    async def test_list(self, mock_async_client):
        resource = AsyncFavorites(mock_async_client)
        params = {"page": 1, "limit": 10}
        await resource.list(params)
        mock_async_client.get_paginated.assert_called_once_with(
            "/v1/me/favorites", query=params
        )

    async def test_list_no_params(self, mock_async_client):
        resource = AsyncFavorites(mock_async_client)
        await resource.list()
        mock_async_client.get_paginated.assert_called_once_with(
            "/v1/me/favorites", query=None
        )

    async def test_add(self, mock_async_client):
        resource = AsyncFavorites(mock_async_client)
        await resource.add("glock-g17")
        mock_async_client.post.assert_called_once_with(
            f"/v1/me/favorites/{quote('glock-g17', safe='')}"
        )

    async def test_remove(self, mock_async_client):
        resource = AsyncFavorites(mock_async_client)
        await resource.remove("glock-g17")
        mock_async_client.delete.assert_called_once_with(
            f"/v1/me/favorites/{quote('glock-g17', safe='')}"
        )



from __future__ import annotations

from typing import TYPE_CHECKING, Dict
from urllib.parse import quote

if TYPE_CHECKING:
    from .._core._http_client import APIResponse, AsyncHttpClient, PaginatedResponse, SyncHttpClient


class Categories:
    """Sync categories resource."""

    def __init__(self, client: SyncHttpClient) -> None:
        self._client = client

    def list(self) -> APIResponse:
        return self._client.get("/v1/categories")

    def get_firearms(self, slug: str, params: Dict | None = None) -> PaginatedResponse:
        return self._client.get_paginated(
            f"/v1/categories/{quote(slug, safe='')}/firearms", query=params
        )


class AsyncCategories:
    """Async categories resource."""

    def __init__(self, client: AsyncHttpClient) -> None:
        self._client = client

    async def list(self) -> APIResponse:
        return await self._client.get("/v1/categories")

    async def get_firearms(self, slug: str, params: Dict | None = None) -> PaginatedResponse:
        return await self._client.get_paginated(
            f"/v1/categories/{quote(slug, safe='')}/firearms", query=params
        )



"""Tests for the Support and AsyncSupport resource classes."""

from __future__ import annotations

from urllib.parse import quote

from gunspec._resources._support import AsyncSupport, Support


class TestSupport:
    """Sync Support resource tests."""

    def test_create(self, mock_sync_client):
        resource = Support(mock_sync_client)
        params = {"subject": "Bug report", "message": "Something is wrong"}
        resource.create(params)
        mock_sync_client.post.assert_called_once_with(
            "/v1/me/support", body=params
        )

    def test_list(self, mock_sync_client):
        resource = Support(mock_sync_client)
        params = {"page": 1, "limit": 10}
        resource.list(params)
        mock_sync_client.get_paginated.assert_called_once_with(
            "/v1/me/support", query=params
        )

    def test_list_no_params(self, mock_sync_client):
        resource = Support(mock_sync_client)
        resource.list()
        mock_sync_client.get_paginated.assert_called_once_with(
            "/v1/me/support", query=None
        )

    def test_get(self, mock_sync_client):
        resource = Support(mock_sync_client)
        resource.get("ticket-123")
        mock_sync_client.get.assert_called_once_with(
            f"/v1/me/support/{quote('ticket-123', safe='')}"
        )

    def test_reply(self, mock_sync_client):
        resource = Support(mock_sync_client)
        params = {"message": "Here is more info"}
        resource.reply("ticket-123", params)
        mock_sync_client.post.assert_called_once_with(
            f"/v1/me/support/{quote('ticket-123', safe='')}/replies", body=params
        )


class TestAsyncSupport:
    """Async Support resource tests."""

    async def test_create(self, mock_async_client):
        resource = AsyncSupport(mock_async_client)
        params = {"subject": "Bug report", "message": "Something is wrong"}
        await resource.create(params)
        mock_async_client.post.assert_called_once_with(
            "/v1/me/support", body=params
        )

    async def test_list(self, mock_async_client):
        resource = AsyncSupport(mock_async_client)
        params = {"page": 1, "limit": 10}
        await resource.list(params)
        mock_async_client.get_paginated.assert_called_once_with(
            "/v1/me/support", query=params
        )

    async def test_list_no_params(self, mock_async_client):
        resource = AsyncSupport(mock_async_client)
        await resource.list()
        mock_async_client.get_paginated.assert_called_once_with(
            "/v1/me/support", query=None
        )

    async def test_get(self, mock_async_client):
        resource = AsyncSupport(mock_async_client)
        await resource.get("ticket-123")
        mock_async_client.get.assert_called_once_with(
            f"/v1/me/support/{quote('ticket-123', safe='')}"
        )

    async def test_reply(self, mock_async_client):
        resource = AsyncSupport(mock_async_client)
        params = {"message": "Here is more info"}
        await resource.reply("ticket-123", params)
        mock_async_client.post.assert_called_once_with(
            f"/v1/me/support/{quote('ticket-123', safe='')}/replies", body=params
        )



from __future__ import annotations

from typing import TYPE_CHECKING, Dict
from urllib.parse import quote

if TYPE_CHECKING:
    from .._core._http_client import APIResponse, AsyncHttpClient, PaginatedResponse, SyncHttpClient


class Support:
    """Sync support resource."""

    def __init__(self, client: SyncHttpClient) -> None:
        self._client = client

    def create(self, params: Dict) -> APIResponse:
        return self._client.post("/v1/me/support", body=params)

    def list(self, params: Dict | None = None) -> PaginatedResponse:
        return self._client.get_paginated("/v1/me/support", query=params)

    def get(self, ticket_id: str) -> APIResponse:
        return self._client.get(f"/v1/me/support/{quote(ticket_id, safe='')}")

    def reply(self, ticket_id: str, params: Dict) -> APIResponse:
        return self._client.post(
            f"/v1/me/support/{quote(ticket_id, safe='')}/replies", body=params
        )


class AsyncSupport:
    """Async support resource."""

    def __init__(self, client: AsyncHttpClient) -> None:
        self._client = client

    async def create(self, params: Dict) -> APIResponse:
        return await self._client.post("/v1/me/support", body=params)

    async def list(self, params: Dict | None = None) -> PaginatedResponse:
        return await self._client.get_paginated("/v1/me/support", query=params)

    async def get(self, ticket_id: str) -> APIResponse:
        return await self._client.get(f"/v1/me/support/{quote(ticket_id, safe='')}")

    async def reply(self, ticket_id: str, params: Dict) -> APIResponse:
        return await self._client.post(
            f"/v1/me/support/{quote(ticket_id, safe='')}/replies", body=params
        )



from __future__ import annotations

from typing import Optional

from ._core._http_client import (
    AsyncHttpClient,
    HttpClientConfig,
    SyncHttpClient,
)
from ._core._retry import RetryConfig
from ._resources import (
    Ammunition,
    AsyncAmmunition,
    AsyncCalibers,
    AsyncCategories,
    AsyncConflicts,
    AsyncCountries,
    AsyncDataQuality,
    AsyncFavorites,
    AsyncFirearms,
    AsyncGame,
    AsyncGameStats_,
    AsyncManufacturers,
    AsyncReports,
    AsyncStats,
    AsyncSupport,
    AsyncUsage,
    AsyncWebhooks,
    Calibers,
    Categories,
    Conflicts,
    Countries,
    DataQuality,
    Favorites,
    Firearms,
    Game,
    GameStats_,
    Manufacturers,
    Reports,
    Stats,
    Support,
    Usage,
    Webhooks,
)
from ._version import __version__


class GunSpec:
    """Synchronous GunSpec.io API client.

    Example::

        from gunspec import GunSpec

        client = GunSpec(api_key="gs_...")

        # List firearms
        result = client.firearms.list({"category": "pistol"})
        for firearm in result.data:
            print(firearm)

        # Context manager for clean shutdown
        with GunSpec() as client:
            data = client.firearms.get("glock-g17")
    """

    firearms: Firearms
    manufacturers: Manufacturers
    calibers: Calibers
    categories: Categories
    stats: Stats
    game: Game
    game_stats: GameStats_
    ammunition: Ammunition
    countries: Countries
    conflicts: Conflicts
    data_quality: DataQuality
    favorites: Favorites
    reports: Reports
    support: Support
    webhooks: Webhooks
    usage: Usage

    def __init__(
        self,
        *,
        api_key: Optional[str] = None,
        base_url: Optional[str] = None,
        timeout: Optional[float] = None,
        retry: Optional[RetryConfig] = None,
        default_headers: Optional[dict[str, str]] = None,
    ) -> None:
        config = HttpClientConfig(
            base_url=base_url or "https://api.gunspec.io",
            timeout=timeout or 30.0,
            api_key=api_key,
            retry=retry or RetryConfig(),
            headers={
                **(default_headers or {}),
                "User-Agent": f"gunspec-sdk/python/{__version__}",
                "X-SDK-Version": __version__,
                "X-SDK-Language": "python",
            },
        )
        self._client = SyncHttpClient(config)

        self.firearms = Firearms(self._client)
        self.manufacturers = Manufacturers(self._client)
        self.calibers = Calibers(self._client)
        self.categories = Categories(self._client)
        self.stats = Stats(self._client)
        self.game = Game(self._client)
        self.game_stats = GameStats_(self._client)
        self.ammunition = Ammunition(self._client)
        self.countries = Countries(self._client)
        self.conflicts = Conflicts(self._client)
        self.data_quality = DataQuality(self._client)
        self.favorites = Favorites(self._client)
        self.reports = Reports(self._client)
        self.support = Support(self._client)
        self.webhooks = Webhooks(self._client)
        self.usage = Usage(self._client)

    def close(self) -> None:
        """Close the underlying HTTP connection pool."""
        self._client.close()

    def __enter__(self) -> GunSpec:
        return self

    def __exit__(self, *_: object) -> None:
        self.close()


class AsyncGunSpec:
    """Asynchronous GunSpec.io API client.

    Example::

        from gunspec import AsyncGunSpec

        async with AsyncGunSpec(api_key="gs_...") as client:
            result = await client.firearms.list({"category": "rifle"})
            for firearm in result.data:
                print(firearm)
    """

    firearms: AsyncFirearms
    manufacturers: AsyncManufacturers
    calibers: AsyncCalibers
    categories: AsyncCategories
    stats: AsyncStats
    game: AsyncGame
    game_stats: AsyncGameStats_
    ammunition: AsyncAmmunition
    countries: AsyncCountries
    conflicts: AsyncConflicts
    data_quality: AsyncDataQuality
    favorites: AsyncFavorites
    reports: AsyncReports
    support: AsyncSupport
    webhooks: AsyncWebhooks
    usage: AsyncUsage

    def __init__(
        self,
        *,
        api_key: Optional[str] = None,
        base_url: Optional[str] = None,
        timeout: Optional[float] = None,
        retry: Optional[RetryConfig] = None,
        default_headers: Optional[dict[str, str]] = None,
    ) -> None:
        config = HttpClientConfig(
            base_url=base_url or "https://api.gunspec.io",
            timeout=timeout or 30.0,
            api_key=api_key,
            retry=retry or RetryConfig(),
            headers={
                **(default_headers or {}),
                "User-Agent": f"gunspec-sdk/python/{__version__}",
                "X-SDK-Version": __version__,
                "X-SDK-Language": "python",
            },
        )
        self._client = AsyncHttpClient(config)

        self.firearms = AsyncFirearms(self._client)
        self.manufacturers = AsyncManufacturers(self._client)
        self.calibers = AsyncCalibers(self._client)
        self.categories = AsyncCategories(self._client)
        self.stats = AsyncStats(self._client)
        self.game = AsyncGame(self._client)
        self.game_stats = AsyncGameStats_(self._client)
        self.ammunition = AsyncAmmunition(self._client)
        self.countries = AsyncCountries(self._client)
        self.conflicts = AsyncConflicts(self._client)
        self.data_quality = AsyncDataQuality(self._client)
        self.favorites = AsyncFavorites(self._client)
        self.reports = AsyncReports(self._client)
        self.support = AsyncSupport(self._client)
        self.webhooks = AsyncWebhooks(self._client)
        self.usage = AsyncUsage(self._client)

    async def aclose(self) -> None:
        """Close the underlying HTTP connection pool."""
        await self._client.aclose()

    async def __aenter__(self) -> AsyncGunSpec:
        return self

    async def __aexit__(self, *_: object) -> None:
        await self.aclose()



"""Shared test fixtures for the GunSpec Python SDK."""

from __future__ import annotations

from typing import Any, Dict, Optional
from unittest.mock import AsyncMock, MagicMock

import pytest

from gunspec._core._http_client import (
    APIResponse,
    PaginatedResponse,
    PaginationMeta,
    RateLimitInfo,
)


def make_api_response(
    data: Any = None,
    status: int = 200,
    request_id: str = "req-test-123",
) -> APIResponse[Any]:
    """Build a mock APIResponse."""
    return APIResponse(
        data=data or {},
        status=status,
        headers={"x-request-id": request_id},
        request_id=request_id,
        rate_limit=RateLimitInfo(limit=100, remaining=99, reset=1700000000),
    )


def make_paginated_response(
    data: Any = None,
    page: int = 1,
    limit: int = 25,
    total: int = 1,
    total_pages: int = 1,
    status: int = 200,
    request_id: str = "req-test-123",
) -> PaginatedResponse[Any]:
    """Build a mock PaginatedResponse."""
    return PaginatedResponse(
        data=data or [],
        pagination=PaginationMeta(
            page=page, limit=limit, total=total, total_pages=total_pages
        ),
        status=status,
        headers={"x-request-id": request_id},
        request_id=request_id,
        rate_limit=RateLimitInfo(limit=100, remaining=99, reset=1700000000),
    )


@pytest.fixture()
def mock_sync_client() -> MagicMock:
    """Return a MagicMock that behaves like SyncHttpClient."""
    client = MagicMock()
    client.get.return_value = make_api_response()
    client.get_paginated.return_value = make_paginated_response()
    client.post.return_value = make_api_response()
    client.put.return_value = make_api_response()
    client.delete.return_value = make_api_response()
    return client


@pytest.fixture()
def mock_async_client() -> AsyncMock:
    """Return an AsyncMock that behaves like AsyncHttpClient."""
    client = AsyncMock()
    client.get.return_value = make_api_response()
    client.get_paginated.return_value = make_paginated_response()
    client.post.return_value = make_api_response()
    client.put.return_value = make_api_response()
    client.delete.return_value = make_api_response()
    return client



from __future__ import annotations

import asyncio
import math
import random
import time
from dataclasses import dataclass
from typing import Callable, Set, TypeVar

from ._errors import (
    APIError,
    ConnectError,
    RateLimitError,
    RequestTimeoutError,
)

T = TypeVar("T")

IDEMPOTENT_METHODS: Set[str] = {"GET", "PUT", "DELETE"}
RETRYABLE_STATUS_CODES: Set[int] = {408, 429, 500, 502, 503, 504}


@dataclass
class RetryConfig:
    """Configuration for retry behaviour with exponential backoff."""

    max_retries: int = 2
    initial_delay_s: float = 0.5
    max_delay_s: float = 8.0
    multiplier: float = 2.0


def is_retryable(error: BaseException, method: str) -> bool:
    """Determine whether the error is eligible for automatic retry."""
    if method.upper() not in IDEMPOTENT_METHODS:
        return False

    if isinstance(error, (ConnectError, RequestTimeoutError)):
        return True

    if isinstance(error, APIError):
        return error.status in RETRYABLE_STATUS_CODES

    return False


def compute_delay(attempt: int, config: RetryConfig, error: BaseException) -> float:
    """Compute the delay in seconds before the next retry attempt.

    Uses exponential backoff with +-20% jitter, capped at ``config.max_delay_s``.
    For 429 responses with a ``Retry-After`` header the returned delay is the
    greater of the computed backoff and the server-requested wait time.
    """
    base = config.initial_delay_s * math.pow(config.multiplier, attempt)
    jitter = 1.0 + (random.random() * 0.4 - 0.2)
    delay = min(base * jitter, config.max_delay_s)

    if isinstance(error, RateLimitError) and error.retry_after is not None:
        delay = max(delay, error.retry_after)

    return round(delay, 3)


def with_retry(
    fn: Callable[[], T],
    method: str,
    config: RetryConfig,
) -> T:
    """Execute a synchronous function with automatic retries on transient failures."""
    last_error: BaseException | None = None

    for attempt in range(config.max_retries + 1):
        try:
            return fn()
        except BaseException as error:
            last_error = error
            is_last = attempt == config.max_retries
            if is_last or not is_retryable(error, method):
                raise
            delay = compute_delay(attempt, config, error)
            time.sleep(delay)

    # Unreachable but satisfies the type checker.
    assert last_error is not None
    raise last_error


async def async_with_retry(
    fn: Callable[[], T],
    method: str,
    config: RetryConfig,
) -> T:
    """Execute an async function with automatic retries on transient failures."""
    last_error: BaseException | None = None

    for attempt in range(config.max_retries + 1):
        try:
            result = fn()
            if asyncio.iscoroutine(result):
                return await result  # type: ignore[misc]
            return result  # type: ignore[return-value]
        except BaseException as error:
            last_error = error
            is_last = attempt == config.max_retries
            if is_last or not is_retryable(error, method):
                raise
            delay = compute_delay(attempt, config, error)
            await asyncio.sleep(delay)

    assert last_error is not None
    raise last_error



"""Tests for gunspec._core._http_client — query serialization, sync/async HTTP clients."""

from __future__ import annotations

import httpx
import pytest
import respx

from gunspec._core._errors import BadRequestError
from gunspec._core._http_client import (
    APIResponse,
    AsyncHttpClient,
    HttpClientConfig,
    PaginatedResponse,
    SyncHttpClient,
    serialize_query,
)
from gunspec._core._retry import RetryConfig


# ---------------------------------------------------------------------------
# serialize_query
# ---------------------------------------------------------------------------


class TestSerializeQuery:
    def test_none_returns_empty_dict(self) -> None:
        assert serialize_query(None) == {}

    def test_none_values_skipped(self) -> None:
        assert serialize_query({"a": None}) == {}

    def test_bool_true_becomes_string(self) -> None:
        assert serialize_query({"flag": True}) == {"flag": "true"}

    def test_bool_false_becomes_string(self) -> None:
        assert serialize_query({"flag": False}) == {"flag": "false"}

    def test_list_preserved(self) -> None:
        assert serialize_query({"ids": [1, 2]}) == {"ids": [1, 2]}

    def test_mixed_params_skip_none(self) -> None:
        result = serialize_query({"name": "test", "x": None})
        assert result == {"name": "test"}

    def test_string_passed_through(self) -> None:
        assert serialize_query({"q": "hello"}) == {"q": "hello"}

    def test_int_passed_through(self) -> None:
        assert serialize_query({"page": 2}) == {"page": 2}

    def test_empty_dict_returns_empty_dict(self) -> None:
        assert serialize_query({}) == {}


# ---------------------------------------------------------------------------
# Helper: standard JSON response bodies
# ---------------------------------------------------------------------------

BASE_URL = "https://api.gunspec.io"

RESPONSE_HEADERS = {
    "X-Request-Id": "req-test-1",
    "X-RateLimit-Limit": "100",
    "X-RateLimit-Remaining": "99",
    "X-RateLimit-Reset": "1700000000",
}

SINGLE_BODY = {"success": True, "data": {"id": 1, "name": "AK-47"}}

PAGINATED_BODY = {
    "success": True,
    "data": [{"id": 1}, {"id": 2}],
    "pagination": {"page": 1, "limit": 25, "total": 50, "totalPages": 2},
}

ERROR_BODY = {
    "success": False,
    "error": {"code": "BAD_REQUEST", "message": "invalid input"},
}

# Use a RetryConfig with 0 retries so tests don't loop.
NO_RETRY = RetryConfig(max_retries=0)


# ---------------------------------------------------------------------------
# SyncHttpClient
# ---------------------------------------------------------------------------


class TestSyncHttpClientGet:
    @respx.mock
    def test_get_returns_api_response_with_unwrapped_data(self) -> None:
        respx.get(f"{BASE_URL}/v1/firearms/1").mock(
            return_value=httpx.Response(200, json=SINGLE_BODY, headers=RESPONSE_HEADERS)
        )
        config = HttpClientConfig(base_url=BASE_URL, retry=NO_RETRY)
        client = SyncHttpClient(config)
        try:
            result = client.get("/v1/firearms/1")
            assert isinstance(result, APIResponse)
            assert result.data == {"id": 1, "name": "AK-47"}
            assert result.status == 200
            assert result.request_id == "req-test-1"
        finally:
            client.close()

    @respx.mock
    def test_get_paginated_returns_paginated_response(self) -> None:
        respx.get(f"{BASE_URL}/v1/firearms").mock(
            return_value=httpx.Response(200, json=PAGINATED_BODY, headers=RESPONSE_HEADERS)
        )
        config = HttpClientConfig(base_url=BASE_URL, retry=NO_RETRY)
        client = SyncHttpClient(config)
        try:
            result = client.get_paginated("/v1/firearms")
            assert isinstance(result, PaginatedResponse)
            assert len(result.data) == 2
            assert result.pagination.page == 1
            assert result.pagination.total == 50
            assert result.pagination.total_pages == 2
        finally:
            client.close()

    @respx.mock
    def test_parses_rate_limit_headers(self) -> None:
        respx.get(f"{BASE_URL}/v1/firearms/1").mock(
            return_value=httpx.Response(200, json=SINGLE_BODY, headers=RESPONSE_HEADERS)
        )
        config = HttpClientConfig(base_url=BASE_URL, retry=NO_RETRY)
        client = SyncHttpClient(config)
        try:
            result = client.get("/v1/firearms/1")
            assert result.rate_limit.limit == 100
            assert result.rate_limit.remaining == 99
            assert result.rate_limit.reset == 1700000000
        finally:
            client.close()

    @respx.mock
    def test_raises_error_on_non_success_status(self) -> None:
        respx.get(f"{BASE_URL}/v1/firearms/bad").mock(
            return_value=httpx.Response(400, json=ERROR_BODY, headers=RESPONSE_HEADERS)
        )
        config = HttpClientConfig(base_url=BASE_URL, retry=NO_RETRY)
        client = SyncHttpClient(config)
        try:
            with pytest.raises(BadRequestError) as exc_info:
                client.get("/v1/firearms/bad")
            assert exc_info.value.status == 400
            assert exc_info.value.code == "BAD_REQUEST"
        finally:
            client.close()

    @respx.mock
    def test_sends_api_key_header(self) -> None:
        route = respx.get(f"{BASE_URL}/v1/firearms/1").mock(
            return_value=httpx.Response(200, json=SINGLE_BODY, headers=RESPONSE_HEADERS)
        )
        config = HttpClientConfig(
            base_url=BASE_URL, api_key="gs_my_key", retry=NO_RETRY
        )
        client = SyncHttpClient(config)
        try:
            client.get("/v1/firearms/1")
            assert route.called
            request = route.calls.last.request
            assert request.headers["x-api-key"] == "gs_my_key"
        finally:
            client.close()

    @respx.mock
    def test_sends_user_agent_header(self) -> None:
        route = respx.get(f"{BASE_URL}/v1/firearms/1").mock(
            return_value=httpx.Response(200, json=SINGLE_BODY, headers=RESPONSE_HEADERS)
        )
        config = HttpClientConfig(base_url=BASE_URL, retry=NO_RETRY)
        client = SyncHttpClient(config)
        try:
            client.get("/v1/firearms/1")
            request = route.calls.last.request
            assert "gunspec-sdk/python" in request.headers["user-agent"]
        finally:
            client.close()


# ---------------------------------------------------------------------------
# AsyncHttpClient
# ---------------------------------------------------------------------------


class TestAsyncHttpClientGet:
    @respx.mock
    async def test_get_returns_api_response(self) -> None:
        respx.get(f"{BASE_URL}/v1/firearms/1").mock(
            return_value=httpx.Response(200, json=SINGLE_BODY, headers=RESPONSE_HEADERS)
        )
        config = HttpClientConfig(base_url=BASE_URL, retry=NO_RETRY)
        client = AsyncHttpClient(config)
        try:
            result = await client.get("/v1/firearms/1")
            assert isinstance(result, APIResponse)
            assert result.data == {"id": 1, "name": "AK-47"}
            assert result.status == 200
            assert result.request_id == "req-test-1"
        finally:
            await client.aclose()

    @respx.mock
    async def test_get_paginated_returns_paginated_response(self) -> None:
        respx.get(f"{BASE_URL}/v1/firearms").mock(
            return_value=httpx.Response(200, json=PAGINATED_BODY, headers=RESPONSE_HEADERS)
        )
        config = HttpClientConfig(base_url=BASE_URL, retry=NO_RETRY)
        client = AsyncHttpClient(config)
        try:
            result = await client.get_paginated("/v1/firearms")
            assert isinstance(result, PaginatedResponse)
            assert len(result.data) == 2
            assert result.pagination.page == 1
            assert result.pagination.total == 50
        finally:
            await client.aclose()

    @respx.mock
    async def test_sends_api_key_header(self) -> None:
        route = respx.get(f"{BASE_URL}/v1/firearms/1").mock(
            return_value=httpx.Response(200, json=SINGLE_BODY, headers=RESPONSE_HEADERS)
        )
        config = HttpClientConfig(
            base_url=BASE_URL, api_key="gs_async_key", retry=NO_RETRY
        )
        client = AsyncHttpClient(config)
        try:
            await client.get("/v1/firearms/1")
            request = route.calls.last.request
            assert request.headers["x-api-key"] == "gs_async_key"
        finally:
            await client.aclose()

    @respx.mock
    async def test_sends_user_agent_header(self) -> None:
        route = respx.get(f"{BASE_URL}/v1/firearms/1").mock(
            return_value=httpx.Response(200, json=SINGLE_BODY, headers=RESPONSE_HEADERS)
        )
        config = HttpClientConfig(base_url=BASE_URL, retry=NO_RETRY)
        client = AsyncHttpClient(config)
        try:
            await client.get("/v1/firearms/1")
            request = route.calls.last.request
            assert "gunspec-sdk/python" in request.headers["user-agent"]
        finally:
            await client.aclose()

    @respx.mock
    async def test_raises_error_on_non_success_status(self) -> None:
        respx.get(f"{BASE_URL}/v1/firearms/bad").mock(
            return_value=httpx.Response(400, json=ERROR_BODY, headers=RESPONSE_HEADERS)
        )
        config = HttpClientConfig(base_url=BASE_URL, retry=NO_RETRY)
        client = AsyncHttpClient(config)
        try:
            with pytest.raises(BadRequestError) as exc_info:
                await client.get("/v1/firearms/bad")
            assert exc_info.value.status == 400
        finally:
            await client.aclose()



"""GunSpec SDK type definitions — models, params, and errors."""

from gunspec.types.errors import APIErrorResponse as APIErrorResponse
from gunspec.types.errors import ErrorDetail as ErrorDetail
from gunspec.types.models import ActionTypeStats as ActionTypeStats
from gunspec.types.models import AdoptionByCountryItem as AdoptionByCountryItem
from gunspec.types.models import AdoptionByTypeItem as AdoptionByTypeItem
from gunspec.types.models import AdoptionMap as AdoptionMap
from gunspec.types.models import AdoptionMapCountry as AdoptionMapCountry
from gunspec.types.models import AdoptionMapUser as AdoptionMapUser
from gunspec.types.models import Ammunition as Ammunition
from gunspec.types.models import BalanceDeviation as BalanceDeviation
from gunspec.types.models import BalanceEntry as BalanceEntry
from gunspec.types.models import BalanceReport as BalanceReport
from gunspec.types.models import BallisticProfile as BallisticProfile
from gunspec.types.models import BallisticProfileAmmo as BallisticProfileAmmo
from gunspec.types.models import BallisticsResult as BallisticsResult
from gunspec.types.models import Caliber as Caliber
from gunspec.types.models import CaliberBallistics as CaliberBallistics
from gunspec.types.models import CaliberBallisticsCaliber as CaliberBallisticsCaliber
from gunspec.types.models import CaliberComparison as CaliberComparison
from gunspec.types.models import CaliberFamily as CaliberFamily
from gunspec.types.models import (
    CaliberPopularityByEra as CaliberPopularityByEra,
)
from gunspec.types.models import (
    CaliberPopularityByEraCaliberEntry as CaliberPopularityByEraCaliberEntry,
)
from gunspec.types.models import Category as Category
from gunspec.types.models import CategoryStats as CategoryStats
from gunspec.types.models import ConfidenceEntry as ConfidenceEntry
from gunspec.types.models import Conflict as Conflict
from gunspec.types.models import ConflictFirearm as ConflictFirearm
from gunspec.types.models import Country as Country
from gunspec.types.models import CountryArsenal as CountryArsenal
from gunspec.types.models import CountryArsenalGroup as CountryArsenalGroup
from gunspec.types.models import DataCoverage as DataCoverage
from gunspec.types.models import DataReport as DataReport
from gunspec.types.models import Dimensions as Dimensions
from gunspec.types.models import DimensionsImperial as DimensionsImperial
from gunspec.types.models import DimensionsMetric as DimensionsMetric
from gunspec.types.models import EraStats as EraStats
from gunspec.types.models import FamilyTree as FamilyTree
from gunspec.types.models import Favorite as Favorite
from gunspec.types.models import FavoriteToggle as FavoriteToggle
from gunspec.types.models import FeatureFrequency as FeatureFrequency
from gunspec.types.models import FieldCoverage as FieldCoverage
from gunspec.types.models import FilterOptionCategory as FilterOptionCategory
from gunspec.types.models import FilterOptionItem as FilterOptionItem
from gunspec.types.models import FilterOptions as FilterOptions
from gunspec.types.models import Firearm as Firearm
from gunspec.types.models import FirearmCaliberEntry as FirearmCaliberEntry
from gunspec.types.models import FirearmCalculation as FirearmCalculation
from gunspec.types.models import FirearmCalculationAmmo as FirearmCalculationAmmo
from gunspec.types.models import (
    FirearmCalculationCalculated as FirearmCalculationCalculated,
)
from gunspec.types.models import FirearmCalculationDelta as FirearmCalculationDelta
from gunspec.types.models import (
    FirearmCalculationFirearm as FirearmCalculationFirearm,
)
from gunspec.types.models import (
    FirearmCalculationSourceReported as FirearmCalculationSourceReported,
)
from gunspec.types.models import FirearmComparison as FirearmComparison
from gunspec.types.models import FirearmDetail as FirearmDetail
from gunspec.types.models import FirearmImage as FirearmImage
from gunspec.types.models import FirearmListItem as FirearmListItem
from gunspec.types.models import FirearmLoadProfile as FirearmLoadProfile
from gunspec.types.models import FirearmLoadProfileAmmo as FirearmLoadProfileAmmo
from gunspec.types.models import (
    FirearmLoadProfileCalculated as FirearmLoadProfileCalculated,
)
from gunspec.types.models import (
    FirearmLoadProfileSourceReported as FirearmLoadProfileSourceReported,
)
from gunspec.types.models import FirearmUser as FirearmUser
from gunspec.types.models import FirearmVariant as FirearmVariant
from gunspec.types.models import GameMetaItem as GameMetaItem
from gunspec.types.models import GameProfile as GameProfile
from gunspec.types.models import GameStats as GameStats
from gunspec.types.models import GameStatsSnapshotEntry as GameStatsSnapshotEntry
from gunspec.types.models import GameStatsVersion as GameStatsVersion
from gunspec.types.models import HeadToHead as HeadToHead
from gunspec.types.models import HeadToHeadVerdict as HeadToHeadVerdict
from gunspec.types.models import ManufacturerStats as ManufacturerStats
from gunspec.types.models import (
    ManufacturerStatsAggregates as ManufacturerStatsAggregates,
)
from gunspec.types.models import (
    ManufacturerStatsCategory as ManufacturerStatsCategory,
)
from gunspec.types.models import (
    ManufacturerStatsManufacturer as ManufacturerStatsManufacturer,
)
from gunspec.types.models import ManufacturerTimeline as ManufacturerTimeline
from gunspec.types.models import (
    ManufacturerTimelineFirearm as ManufacturerTimelineFirearm,
)
from gunspec.types.models import (
    ManufacturerTimelineYear as ManufacturerTimelineYear,
)
from gunspec.types.models import Manufacturer as Manufacturer
from gunspec.types.models import MatchupFirearm as MatchupFirearm
from gunspec.types.models import MatchupResult as MatchupResult
from gunspec.types.models import MaterialStats as MaterialStats
from gunspec.types.models import MaterialStatsEntry as MaterialStatsEntry
from gunspec.types.models import PaginatedResponseModel as PaginatedResponseModel
from gunspec.types.models import PaginationMeta as PaginationMeta
from gunspec.types.models import PopularCaliber as PopularCaliber
from gunspec.types.models import PowerRating as PowerRating
from gunspec.types.models import PowerRatingBreakdown as PowerRatingBreakdown
from gunspec.types.models import ProductionStatusItem as ProductionStatusItem
from gunspec.types.models import ProlificManufacturer as ProlificManufacturer
from gunspec.types.models import RoleRoster as RoleRoster
from gunspec.types.models import RoleRosterItem as RoleRosterItem
from gunspec.types.models import Silhouette as Silhouette
from gunspec.types.models import SimilarFirearm as SimilarFirearm
from gunspec.types.models import StatDistribution as StatDistribution
from gunspec.types.models import (
    StatDistributionHistogramBucket as StatDistributionHistogramBucket,
)
from gunspec.types.models import (
    StatDistributionPercentiles as StatDistributionPercentiles,
)
from gunspec.types.models import StatsSummary as StatsSummary
from gunspec.types.models import SuccessResponseModel as SuccessResponseModel
from gunspec.types.models import SupportTicket as SupportTicket
from gunspec.types.models import SupportTicketDetail as SupportTicketDetail
from gunspec.types.models import SupportTicketReply as SupportTicketReply
from gunspec.types.models import TerminalBallistics as TerminalBallistics
from gunspec.types.models import TierItem as TierItem
from gunspec.types.models import TierList as TierList
from gunspec.types.models import TierListTiers as TierListTiers
from gunspec.types.models import TimelineItem as TimelineItem
from gunspec.types.models import TopFirearmItem as TopFirearmItem
from gunspec.types.models import TrajectoryPoint as TrajectoryPoint
from gunspec.types.models import UsageStats as UsageStats
from gunspec.types.models import UsageStatsCurrentMonth as UsageStatsCurrentMonth
from gunspec.types.models import UsageStatsDailyEntry as UsageStatsDailyEntry
from gunspec.types.models import UsageStatsPerKeyEntry as UsageStatsPerKeyEntry
from gunspec.types.models import UsageStatsTier as UsageStatsTier
from gunspec.types.models import WebhookEndpoint as WebhookEndpoint
from gunspec.types.models import WebhookTestResult as WebhookTestResult
from gunspec.types.params import ActionTypesParams as ActionTypesParams
from gunspec.types.params import AdoptionByCountryParams as AdoptionByCountryParams
from gunspec.types.params import AdoptionByTypeParams as AdoptionByTypeParams
from gunspec.types.params import AmmunitionBallisticsParams as AmmunitionBallisticsParams
from gunspec.types.params import BalanceReportParams as BalanceReportParams
from gunspec.types.params import ByActionParams as ByActionParams
from gunspec.types.params import ByConflictParams as ByConflictParams
from gunspec.types.params import ByDesignerParams as ByDesignerParams
from gunspec.types.params import ByEraParams as ByEraParams
from gunspec.types.params import ByFeatureParams as ByFeatureParams
from gunspec.types.params import ByMaterialParams as ByMaterialParams
from gunspec.types.params import (
    CalculateBallisticsParams as CalculateBallisticsParams,
)
from gunspec.types.params import CaliberBallisticsParams as CaliberBallisticsParams
from gunspec.types.params import (
    CaliberPopularityByEraParams as CaliberPopularityByEraParams,
)
from gunspec.types.params import CompareCalibersParams as CompareCalibersParams
from gunspec.types.params import CompareFirearmsParams as CompareFirearmsParams
from gunspec.types.params import ConfidenceParams as ConfidenceParams
from gunspec.types.params import ConflictNameQuery as ConflictNameQuery
from gunspec.types.params import CountryCodeParam as CountryCodeParam
from gunspec.types.params import CreateReplyParams as CreateReplyParams
from gunspec.types.params import CreateReportParams as CreateReportParams
from gunspec.types.params import CreateTicketParams as CreateTicketParams
from gunspec.types.params import (
    CreateWebhookEndpointParams as CreateWebhookEndpointParams,
)
from gunspec.types.params import FeatureFrequencyParams as FeatureFrequencyParams
from gunspec.types.params import GameMetaParams as GameMetaParams
from gunspec.types.params import HeadToHeadParams as HeadToHeadParams
from gunspec.types.params import ListAmmunitionParams as ListAmmunitionParams
from gunspec.types.params import ListCalibersParams as ListCalibersParams
from gunspec.types.params import ListFavoritesParams as ListFavoritesParams
from gunspec.types.params import ListFirearmsParams as ListFirearmsParams
from gunspec.types.params import ListManufacturersParams as ListManufacturersParams
from gunspec.types.params import ListReportsParams as ListReportsParams
from gunspec.types.params import (
    ListSnapshotFirearmsParams as ListSnapshotFirearmsParams,
)
from gunspec.types.params import ListTicketsParams as ListTicketsParams
from gunspec.types.params import (
    ListWebhookEndpointsParams as ListWebhookEndpointsParams,
)
from gunspec.types.params import LoadFirearmParams as LoadFirearmParams
from gunspec.types.params import MatchupsParams as MatchupsParams
from gunspec.types.params import PaginationParams as PaginationParams
from gunspec.types.params import PopularCalibersParams as PopularCalibersParams
from gunspec.types.params import PowerRatingParams as PowerRatingParams
from gunspec.types.params import (
    ProlificManufacturersParams as ProlificManufacturersParams,
)
from gunspec.types.params import RandomFirearmParams as RandomFirearmParams
from gunspec.types.params import RoleRosterParams as RoleRosterParams
from gunspec.types.params import SearchFirearmsParams as SearchFirearmsParams
from gunspec.types.params import SilhouetteParams as SilhouetteParams
from gunspec.types.params import StatDistributionParams as StatDistributionParams
from gunspec.types.params import TierListParams as TierListParams
from gunspec.types.params import TimelineParams as TimelineParams
from gunspec.types.params import TopFirearmsParams as TopFirearmsParams
from gunspec.types.params import (
    UpdateWebhookEndpointParams as UpdateWebhookEndpointParams,
)
from gunspec.types.params import UsageParams as UsageParams

__all__ = [
    # Errors
    "APIErrorResponse",
    "ErrorDetail",
    # Models — Firearms
    "Firearm",
    "FirearmListItem",
    "FirearmCaliberEntry",
    "FirearmDetail",
    "FirearmVariant",
    # Models — Manufacturers
    "Manufacturer",
    "ManufacturerStats",
    "ManufacturerStatsManufacturer",
    "ManufacturerStatsAggregates",
    "ManufacturerStatsCategory",
    "ManufacturerTimeline",
    "ManufacturerTimelineFirearm",
    "ManufacturerTimelineYear",
    # Models — Calibers
    "Caliber",
    "CaliberBallistics",
    "CaliberBallisticsCaliber",
    "CaliberFamily",
    # Models — Ammunition
    "Ammunition",
    "TrajectoryPoint",
    "TerminalBallistics",
    "BallisticProfile",
    "BallisticProfileAmmo",
    "FirearmCalculation",
    "FirearmCalculationFirearm",
    "FirearmCalculationAmmo",
    "FirearmCalculationCalculated",
    "FirearmCalculationSourceReported",
    "FirearmCalculationDelta",
    "FirearmLoadProfile",
    "FirearmLoadProfileAmmo",
    "FirearmLoadProfileCalculated",
    "FirearmLoadProfileSourceReported",
    # Models — Categories
    "Category",
    # Models — Images & Users
    "FirearmImage",
    "FirearmUser",
    # Models — Game Stats
    "GameStats",
    "GameProfile",
    "GameMetaItem",
    # Models — Game Endpoints
    "BalanceDeviation",
    "BalanceEntry",
    "TierItem",
    "TierList",
    "TierListTiers",
    "MatchupFirearm",
    "MatchupResult",
    "RoleRosterItem",
    "StatDistribution",
    "StatDistributionPercentiles",
    "StatDistributionHistogramBucket",
    # Models — Game Stats Snapshots
    "GameStatsVersion",
    "GameStatsSnapshotEntry",
    # Models — Comparisons
    "FirearmComparison",
    "CaliberComparison",
    "HeadToHead",
    "HeadToHeadVerdict",
    # Models — Family Tree
    "FamilyTree",
    # Models — Similar
    "SimilarFirearm",
    # Models — Adoption
    "AdoptionMap",
    "AdoptionMapCountry",
    "AdoptionMapUser",
    # Models — Top / Power / Timeline
    "TopFirearmItem",
    "PowerRating",
    "PowerRatingBreakdown",
    "TimelineItem",
    # Models — Dimensions
    "Dimensions",
    "DimensionsMetric",
    "DimensionsImperial",
    # Models — Filter Options
    "FilterOptions",
    "FilterOptionCategory",
    "FilterOptionItem",
    # Models — Statistics
    "StatsSummary",
    "ProductionStatusItem",
    "FieldCoverage",
    "PopularCaliber",
    "ProlificManufacturer",
    "CategoryStats",
    "EraStats",
    "MaterialStats",
    "MaterialStatsEntry",
    "AdoptionByCountryItem",
    "AdoptionByTypeItem",
    "ActionTypeStats",
    "FeatureFrequency",
    "CaliberPopularityByEra",
    "CaliberPopularityByEraCaliberEntry",
    # Models — Countries & Conflicts
    "Country",
    "CountryArsenal",
    "CountryArsenalGroup",
    "Conflict",
    "ConflictFirearm",
    # Models — Data Quality
    "DataCoverage",
    "ConfidenceEntry",
    # Models — Ballistics & Silhouette
    "BallisticsResult",
    "Silhouette",
    # Models — Balance & Roster
    "BalanceReport",
    "RoleRoster",
    # Models — User Resources
    "Favorite",
    "FavoriteToggle",
    "DataReport",
    "SupportTicket",
    "SupportTicketDetail",
    "SupportTicketReply",
    "WebhookEndpoint",
    "WebhookTestResult",
    "UsageStats",
    "UsageStatsCurrentMonth",
    "UsageStatsDailyEntry",
    "UsageStatsPerKeyEntry",
    "UsageStatsTier",
    # Models — Response Wrappers
    "PaginatedResponseModel",
    "PaginationMeta",
    "SuccessResponseModel",
    # Params
    "PaginationParams",
    "ListFirearmsParams",
    "SearchFirearmsParams",
    "CompareFirearmsParams",
    "GameMetaParams",
    "RandomFirearmParams",
    "TopFirearmsParams",
    "HeadToHeadParams",
    "ByFeatureParams",
    "ByActionParams",
    "ByMaterialParams",
    "ByDesignerParams",
    "PowerRatingParams",
    "TimelineParams",
    "ByConflictParams",
    "CalculateBallisticsParams",
    "LoadFirearmParams",
    "ListManufacturersParams",
    "ListCalibersParams",
    "CompareCalibersParams",
    "CaliberBallisticsParams",
    "ListAmmunitionParams",
    "AmmunitionBallisticsParams",
    "PopularCalibersParams",
    "ProlificManufacturersParams",
    "ByEraParams",
    "AdoptionByCountryParams",
    "AdoptionByTypeParams",
    "ActionTypesParams",
    "FeatureFrequencyParams",
    "CaliberPopularityByEraParams",
    "BalanceReportParams",
    "TierListParams",
    "MatchupsParams",
    "RoleRosterParams",
    "StatDistributionParams",
    "ListSnapshotFirearmsParams",
    "CountryCodeParam",
    "ConflictNameQuery",
    "ConfidenceParams",
    "SilhouetteParams",
    "ListFavoritesParams",
    "CreateReportParams",
    "ListReportsParams",
    "CreateTicketParams",
    "ListTicketsParams",
    "CreateReplyParams",
    "CreateWebhookEndpointParams",
    "UpdateWebhookEndpointParams",
    "ListWebhookEndpointsParams",
    "UsageParams",
]



"""GunSpec.io Python SDK."""

from __future__ import annotations

from gunspec._client import AsyncGunSpec, GunSpec
from gunspec._core._errors import (
    APIError,
    AuthenticationError,
    BadRequestError,
    ConnectError,
    GunSpecError,
    InternalServerError,
    NotFoundError,
    PermissionDeniedError,
    RateLimitError,
    RequestTimeoutError,
)
from gunspec._core._http_client import (
    APIResponse,
    HttpClientConfig,
    PaginatedResponse,
    PaginationMeta,
    RateLimitInfo,
)
from gunspec._core._pagination import AsyncPage, SyncPage
from gunspec._core._retry import RetryConfig
from gunspec._version import __version__

__all__ = [
    # Client
    "GunSpec",
    "AsyncGunSpec",
    # Version
    "__version__",
    # Config
    "HttpClientConfig",
    "RetryConfig",
    # Response types
    "APIResponse",
    "PaginatedResponse",
    "PaginationMeta",
    "RateLimitInfo",
    # Pagination
    "SyncPage",
    "AsyncPage",
    # Errors
    "GunSpecError",
    "APIError",
    "AuthenticationError",
    "PermissionDeniedError",
    "NotFoundError",
    "BadRequestError",
    "RateLimitError",
    "InternalServerError",
    "ConnectError",
    "RequestTimeoutError",
]



from __future__ import annotations

from typing import TYPE_CHECKING, AsyncIterator, Dict, Iterator
from urllib.parse import quote

if TYPE_CHECKING:
    from .._core._http_client import APIResponse, AsyncHttpClient, PaginatedResponse, SyncHttpClient


class Calibers:
    """Sync calibers resource."""

    def __init__(self, client: SyncHttpClient) -> None:
        self._client = client

    def list(self, params: Dict | None = None) -> PaginatedResponse:
        return self._client.get_paginated("/v1/calibers", query=params)

    def list_auto_paging(self, params: Dict | None = None) -> Iterator:
        page = (params or {}).get("page", 1)
        while True:
            result = self.list({**(params or {}), "page": page})
            yield from result.data
            if not result.pagination.total_pages or page >= result.pagination.total_pages:
                break
            page += 1

    def compare(self, params: Dict) -> APIResponse:
        return self._client.get("/v1/calibers/compare", query=params)

    def ballistics(self, params: Dict) -> APIResponse:
        return self._client.get("/v1/calibers/ballistics", query=params)

    def get(self, id: str) -> APIResponse:
        return self._client.get(f"/v1/calibers/{quote(id, safe='')}")

    def get_firearms(self, id: str, params: Dict | None = None) -> PaginatedResponse:
        return self._client.get_paginated(
            f"/v1/calibers/{quote(id, safe='')}/firearms", query=params
        )

    def get_parent_chain(self, id: str) -> APIResponse:
        return self._client.get(f"/v1/calibers/{quote(id, safe='')}/parent-chain")

    def get_family(self, id: str) -> APIResponse:
        return self._client.get(f"/v1/calibers/{quote(id, safe='')}/family")

    def get_ammunition(self, id: str, params: Dict | None = None) -> PaginatedResponse:
        return self._client.get_paginated(
            f"/v1/calibers/{quote(id, safe='')}/ammunition", query=params
        )


class AsyncCalibers:
    """Async calibers resource."""

    def __init__(self, client: AsyncHttpClient) -> None:
        self._client = client

    async def list(self, params: Dict | None = None) -> PaginatedResponse:
        return await self._client.get_paginated("/v1/calibers", query=params)

    async def list_auto_paging(self, params: Dict | None = None) -> AsyncIterator:
        page = (params or {}).get("page", 1)
        while True:
            result = await self.list({**(params or {}), "page": page})
            for item in result.data:
                yield item
            if not result.pagination.total_pages or page >= result.pagination.total_pages:
                break
            page += 1

    async def compare(self, params: Dict) -> APIResponse:
        return await self._client.get("/v1/calibers/compare", query=params)

    async def ballistics(self, params: Dict) -> APIResponse:
        return await self._client.get("/v1/calibers/ballistics", query=params)

    async def get(self, id: str) -> APIResponse:
        return await self._client.get(f"/v1/calibers/{quote(id, safe='')}")

    async def get_firearms(self, id: str, params: Dict | None = None) -> PaginatedResponse:
        return await self._client.get_paginated(
            f"/v1/calibers/{quote(id, safe='')}/firearms", query=params
        )

    async def get_parent_chain(self, id: str) -> APIResponse:
        return await self._client.get(f"/v1/calibers/{quote(id, safe='')}/parent-chain")

    async def get_family(self, id: str) -> APIResponse:
        return await self._client.get(f"/v1/calibers/{quote(id, safe='')}/family")

    async def get_ammunition(self, id: str, params: Dict | None = None) -> PaginatedResponse:
        return await self._client.get_paginated(
            f"/v1/calibers/{quote(id, safe='')}/ammunition", query=params
        )



"""Tests for the Conflicts and AsyncConflicts resource classes."""

from __future__ import annotations

from gunspec._resources._conflicts import AsyncConflicts, Conflicts


class TestConflicts:
    """Sync Conflicts resource tests."""

    def test_list(self, mock_sync_client):
        resource = Conflicts(mock_sync_client)
        resource.list()
        mock_sync_client.get.assert_called_once_with("/v1/conflicts")


class TestAsyncConflicts:
    """Async Conflicts resource tests."""

    async def test_list(self, mock_async_client):
        resource = AsyncConflicts(mock_async_client)
        await resource.list()
        mock_async_client.get.assert_called_once_with("/v1/conflicts")



from __future__ import annotations

from typing import TYPE_CHECKING, Dict

if TYPE_CHECKING:
    from .._core._http_client import APIResponse, AsyncHttpClient, SyncHttpClient


class Game:
    """Sync game resource."""

    def __init__(self, client: SyncHttpClient) -> None:
        self._client = client

    def balance_report(self, params: Dict | None = None) -> APIResponse:
        return self._client.get("/v1/game/balance-report", query=params)

    def tier_list(self, params: Dict | None = None) -> APIResponse:
        return self._client.get("/v1/game/tier-list", query=params)

    def matchups(self, params: Dict) -> APIResponse:
        return self._client.get("/v1/game/matchups", query=params)

    def role_roster(self, params: Dict) -> APIResponse:
        return self._client.get("/v1/game/role-roster", query=params)

    def stat_distribution(self, params: Dict) -> APIResponse:
        return self._client.get("/v1/game/stat-distribution", query=params)


class AsyncGame:
    """Async game resource."""

    def __init__(self, client: AsyncHttpClient) -> None:
        self._client = client

    async def balance_report(self, params: Dict | None = None) -> APIResponse:
        return await self._client.get("/v1/game/balance-report", query=params)

    async def tier_list(self, params: Dict | None = None) -> APIResponse:
        return await self._client.get("/v1/game/tier-list", query=params)

    async def matchups(self, params: Dict) -> APIResponse:
        return await self._client.get("/v1/game/matchups", query=params)

    async def role_roster(self, params: Dict) -> APIResponse:
        return await self._client.get("/v1/game/role-roster", query=params)

    async def stat_distribution(self, params: Dict) -> APIResponse:
        return await self._client.get("/v1/game/stat-distribution", query=params)



"""Tests for gunspec._core._retry — retry config, retryability, backoff, and retry loops."""

from __future__ import annotations

from unittest.mock import MagicMock, patch

import pytest

from gunspec._core._errors import (
    APIError,
    BadRequestError,
    ConnectError,
    InternalServerError,
    RateLimitError,
    RequestTimeoutError,
)
from gunspec._core._retry import (
    RetryConfig,
    async_with_retry,
    compute_delay,
    is_retryable,
    with_retry,
)


# ---------------------------------------------------------------------------
# RetryConfig defaults
# ---------------------------------------------------------------------------


class TestRetryConfig:
    def test_defaults(self) -> None:
        cfg = RetryConfig()
        assert cfg.max_retries == 2
        assert cfg.initial_delay_s == 0.5
        assert cfg.max_delay_s == 8.0
        assert cfg.multiplier == 2.0

    def test_custom_values(self) -> None:
        cfg = RetryConfig(max_retries=5, initial_delay_s=1.0, max_delay_s=60.0, multiplier=3.0)
        assert cfg.max_retries == 5
        assert cfg.initial_delay_s == 1.0
        assert cfg.max_delay_s == 60.0
        assert cfg.multiplier == 3.0


# ---------------------------------------------------------------------------
# is_retryable
# ---------------------------------------------------------------------------


class TestIsRetryable:
    def test_500_get_is_retryable(self) -> None:
        err = APIError(500, "ERR", "boom", "req-1", {})
        assert is_retryable(err, "GET") is True

    def test_500_post_not_retryable(self) -> None:
        err = APIError(500, "ERR", "boom", "req-1", {})
        assert is_retryable(err, "POST") is False

    def test_400_get_not_retryable(self) -> None:
        err = BadRequestError("BAD", "invalid", "req-1", {})
        assert is_retryable(err, "GET") is False

    def test_connect_error_get_retryable(self) -> None:
        err = ConnectError("fail")
        assert is_retryable(err, "GET") is True

    def test_connect_error_post_not_retryable(self) -> None:
        err = ConnectError("fail")
        assert is_retryable(err, "POST") is False

    def test_timeout_error_get_retryable(self) -> None:
        err = RequestTimeoutError(30)
        assert is_retryable(err, "GET") is True

    def test_timeout_error_post_not_retryable(self) -> None:
        err = RequestTimeoutError(30)
        assert is_retryable(err, "POST") is False

    def test_429_get_is_retryable(self) -> None:
        err = RateLimitError("RATE", "slow", "req-1", {}, retry_after=1.0)
        assert is_retryable(err, "GET") is True

    def test_put_is_idempotent(self) -> None:
        err = APIError(500, "ERR", "boom", "req-1", {})
        assert is_retryable(err, "PUT") is True

    def test_delete_is_idempotent(self) -> None:
        err = APIError(500, "ERR", "boom", "req-1", {})
        assert is_retryable(err, "DELETE") is True


# ---------------------------------------------------------------------------
# compute_delay
# ---------------------------------------------------------------------------


class TestComputeDelay:
    def test_first_attempt_around_initial_delay(self) -> None:
        """Attempt 0 should produce ~0.5s with jitter in +-20% range."""
        cfg = RetryConfig()
        err = APIError(500, "ERR", "boom", "req-1", {})
        delays = [compute_delay(0, cfg, err) for _ in range(50)]
        for d in delays:
            assert 0.39 <= d <= 0.61, f"delay {d} outside expected jitter range"

    def test_respects_max_delay(self) -> None:
        cfg = RetryConfig(max_delay_s=1.0, initial_delay_s=2.0)
        err = APIError(500, "ERR", "boom", "req-1", {})
        delay = compute_delay(0, cfg, err)
        assert delay <= 1.0

    def test_exponential_growth(self) -> None:
        """Attempt 1 should have roughly double the base of attempt 0."""
        cfg = RetryConfig(max_delay_s=100.0)
        err = APIError(500, "ERR", "boom", "req-1", {})
        # Use fixed random to test base growth
        with patch("gunspec._core._retry.random.random", return_value=0.5):
            d0 = compute_delay(0, cfg, err)
            d1 = compute_delay(1, cfg, err)
        assert d1 == pytest.approx(d0 * cfg.multiplier, rel=0.01)

    def test_rate_limit_retry_after_respected(self) -> None:
        """When retry_after > computed backoff, use retry_after."""
        cfg = RetryConfig()
        err = RateLimitError("RATE", "slow", "req-1", {}, retry_after=10.0)
        delay = compute_delay(0, cfg, err)
        assert delay >= 10.0

    def test_rate_limit_retry_after_not_used_when_smaller(self) -> None:
        """When retry_after < computed backoff, the computed backoff wins."""
        cfg = RetryConfig(initial_delay_s=5.0, max_delay_s=100.0)
        err = RateLimitError("RATE", "slow", "req-1", {}, retry_after=0.1)
        delay = compute_delay(0, cfg, err)
        assert delay >= 0.1  # at least retry_after
        assert delay >= 3.5  # close to initial_delay (5 * 0.8 jitter low)


# ---------------------------------------------------------------------------
# with_retry (synchronous)
# ---------------------------------------------------------------------------


class TestWithRetry:
    def test_returns_result_on_first_success(self) -> None:
        fn = MagicMock(return_value="ok")
        result = with_retry(fn, "GET", RetryConfig())
        assert result == "ok"
        assert fn.call_count == 1

    @patch("gunspec._core._retry.time.sleep")
    def test_retries_on_retryable_error_then_succeeds(self, mock_sleep: MagicMock) -> None:
        fn = MagicMock(
            side_effect=[APIError(500, "ERR", "boom", "req-1", {}), "ok"]
        )
        result = with_retry(fn, "GET", RetryConfig(max_retries=2))
        assert result == "ok"
        assert fn.call_count == 2
        assert mock_sleep.call_count == 1

    @patch("gunspec._core._retry.time.sleep")
    def test_raises_after_max_retries_exhausted(self, mock_sleep: MagicMock) -> None:
        err = APIError(500, "ERR", "boom", "req-1", {})
        fn = MagicMock(side_effect=err)
        with pytest.raises(APIError, match="boom"):
            with_retry(fn, "GET", RetryConfig(max_retries=2))
        assert fn.call_count == 3  # initial + 2 retries

    def test_does_not_retry_non_retryable_errors(self) -> None:
        err = BadRequestError("BAD", "invalid", "req-1", {})
        fn = MagicMock(side_effect=err)
        with pytest.raises(BadRequestError, match="invalid"):
            with_retry(fn, "GET", RetryConfig(max_retries=2))
        assert fn.call_count == 1

    def test_does_not_retry_non_idempotent_methods(self) -> None:
        err = APIError(500, "ERR", "boom", "req-1", {})
        fn = MagicMock(side_effect=err)
        with pytest.raises(APIError, match="boom"):
            with_retry(fn, "POST", RetryConfig(max_retries=2))
        assert fn.call_count == 1


# ---------------------------------------------------------------------------
# async_with_retry
# ---------------------------------------------------------------------------


class TestAsyncWithRetry:
    async def test_returns_result_on_first_success(self) -> None:
        call_count = 0

        async def fn() -> str:
            nonlocal call_count
            call_count += 1
            return "ok"

        result = await async_with_retry(fn, "GET", RetryConfig())
        assert result == "ok"
        assert call_count == 1

    @patch("gunspec._core._retry.asyncio.sleep")
    async def test_retries_on_retryable_error_then_succeeds(
        self, mock_sleep: MagicMock
    ) -> None:
        # Make mock_sleep a proper coroutine
        async def noop_sleep(delay: float) -> None:
            pass

        mock_sleep.side_effect = noop_sleep

        call_count = 0

        async def fn() -> str:
            nonlocal call_count
            call_count += 1
            if call_count == 1:
                raise APIError(500, "ERR", "boom", "req-1", {})
            return "ok"

        result = await async_with_retry(fn, "GET", RetryConfig(max_retries=2))
        assert result == "ok"
        assert call_count == 2
        assert mock_sleep.call_count == 1

    @patch("gunspec._core._retry.asyncio.sleep")
    async def test_raises_after_max_retries_exhausted(
        self, mock_sleep: MagicMock
    ) -> None:
        async def noop_sleep(delay: float) -> None:
            pass

        mock_sleep.side_effect = noop_sleep

        async def fn() -> str:
            raise APIError(500, "ERR", "boom", "req-1", {})

        with pytest.raises(APIError, match="boom"):
            await async_with_retry(fn, "GET", RetryConfig(max_retries=2))



"""Tests for the Stats and AsyncStats resource classes."""

from __future__ import annotations

from gunspec._resources._stats import AsyncStats, Stats


class TestStats:
    """Sync Stats resource tests."""

    def test_summary(self, mock_sync_client):
        resource = Stats(mock_sync_client)
        resource.summary()
        mock_sync_client.get.assert_called_once_with("/v1/stats/summary")

    def test_production_status(self, mock_sync_client):
        resource = Stats(mock_sync_client)
        resource.production_status()
        mock_sync_client.get.assert_called_once_with("/v1/stats/production-status")

    def test_field_coverage(self, mock_sync_client):
        resource = Stats(mock_sync_client)
        resource.field_coverage()
        mock_sync_client.get.assert_called_once_with("/v1/stats/field-coverage")

    def test_popular_calibers(self, mock_sync_client):
        resource = Stats(mock_sync_client)
        params = {"limit": 10}
        resource.popular_calibers(params)
        mock_sync_client.get.assert_called_once_with(
            "/v1/stats/calibers/popular", query=params
        )

    def test_popular_calibers_no_params(self, mock_sync_client):
        resource = Stats(mock_sync_client)
        resource.popular_calibers()
        mock_sync_client.get.assert_called_once_with(
            "/v1/stats/calibers/popular", query=None
        )

    def test_prolific_manufacturers(self, mock_sync_client):
        resource = Stats(mock_sync_client)
        params = {"limit": 5}
        resource.prolific_manufacturers(params)
        mock_sync_client.get.assert_called_once_with(
            "/v1/stats/manufacturers/prolific", query=params
        )

    def test_prolific_manufacturers_no_params(self, mock_sync_client):
        resource = Stats(mock_sync_client)
        resource.prolific_manufacturers()
        mock_sync_client.get.assert_called_once_with(
            "/v1/stats/manufacturers/prolific", query=None
        )

    def test_by_category(self, mock_sync_client):
        resource = Stats(mock_sync_client)
        resource.by_category()
        mock_sync_client.get.assert_called_once_with("/v1/stats/by-category")

    def test_by_era(self, mock_sync_client):
        resource = Stats(mock_sync_client)
        params = {"era": "modern"}
        resource.by_era(params)
        mock_sync_client.get.assert_called_once_with(
            "/v1/stats/by-era", query=params
        )

    def test_materials(self, mock_sync_client):
        resource = Stats(mock_sync_client)
        resource.materials()
        mock_sync_client.get.assert_called_once_with("/v1/stats/materials")

    def test_adoption_by_country(self, mock_sync_client):
        resource = Stats(mock_sync_client)
        params = {"country": "US"}
        resource.adoption_by_country(params)
        mock_sync_client.get.assert_called_once_with(
            "/v1/stats/adoption/by-country", query=params
        )

    def test_adoption_by_type(self, mock_sync_client):
        resource = Stats(mock_sync_client)
        params = {"type": "military"}
        resource.adoption_by_type(params)
        mock_sync_client.get.assert_called_once_with(
            "/v1/stats/adoption/by-type", query=params
        )

    def test_action_types(self, mock_sync_client):
        resource = Stats(mock_sync_client)
        params = {"category": "pistol"}
        resource.action_types(params)
        mock_sync_client.get.assert_called_once_with(
            "/v1/stats/action-types", query=params
        )

    def test_action_types_no_params(self, mock_sync_client):
        resource = Stats(mock_sync_client)
        resource.action_types()
        mock_sync_client.get.assert_called_once_with(
            "/v1/stats/action-types", query=None
        )

    def test_feature_frequency(self, mock_sync_client):
        resource = Stats(mock_sync_client)
        params = {"min_count": 5}
        resource.feature_frequency(params)
        mock_sync_client.get.assert_called_once_with(
            "/v1/stats/feature-frequency", query=params
        )

    def test_feature_frequency_no_params(self, mock_sync_client):
        resource = Stats(mock_sync_client)
        resource.feature_frequency()
        mock_sync_client.get.assert_called_once_with(
            "/v1/stats/feature-frequency", query=None
        )

    def test_caliber_popularity_by_era(self, mock_sync_client):
        resource = Stats(mock_sync_client)
        params = {"era": "cold-war"}
        resource.caliber_popularity_by_era(params)
        mock_sync_client.get.assert_called_once_with(
            "/v1/stats/caliber-popularity-by-era", query=params
        )

    def test_caliber_popularity_by_era_no_params(self, mock_sync_client):
        resource = Stats(mock_sync_client)
        resource.caliber_popularity_by_era()
        mock_sync_client.get.assert_called_once_with(
            "/v1/stats/caliber-popularity-by-era", query=None
        )


class TestAsyncStats:
    """Async Stats resource tests."""

    async def test_summary(self, mock_async_client):
        resource = AsyncStats(mock_async_client)
        await resource.summary()
        mock_async_client.get.assert_called_once_with("/v1/stats/summary")

    async def test_production_status(self, mock_async_client):
        resource = AsyncStats(mock_async_client)
        await resource.production_status()
        mock_async_client.get.assert_called_once_with("/v1/stats/production-status")

    async def test_field_coverage(self, mock_async_client):
        resource = AsyncStats(mock_async_client)
        await resource.field_coverage()
        mock_async_client.get.assert_called_once_with("/v1/stats/field-coverage")

    async def test_popular_calibers(self, mock_async_client):
        resource = AsyncStats(mock_async_client)
        params = {"limit": 10}
        await resource.popular_calibers(params)
        mock_async_client.get.assert_called_once_with(
            "/v1/stats/calibers/popular", query=params
        )

    async def test_prolific_manufacturers(self, mock_async_client):
        resource = AsyncStats(mock_async_client)
        params = {"limit": 5}
        await resource.prolific_manufacturers(params)
        mock_async_client.get.assert_called_once_with(
            "/v1/stats/manufacturers/prolific", query=params
        )

    async def test_by_category(self, mock_async_client):
        resource = AsyncStats(mock_async_client)
        await resource.by_category()
        mock_async_client.get.assert_called_once_with("/v1/stats/by-category")

    async def test_by_era(self, mock_async_client):
        resource = AsyncStats(mock_async_client)
        params = {"era": "modern"}
        await resource.by_era(params)
        mock_async_client.get.assert_called_once_with(
            "/v1/stats/by-era", query=params
        )

    async def test_materials(self, mock_async_client):
        resource = AsyncStats(mock_async_client)
        await resource.materials()
        mock_async_client.get.assert_called_once_with("/v1/stats/materials")

    async def test_adoption_by_country(self, mock_async_client):
        resource = AsyncStats(mock_async_client)
        params = {"country": "US"}
        await resource.adoption_by_country(params)
        mock_async_client.get.assert_called_once_with(
            "/v1/stats/adoption/by-country", query=params
        )

    async def test_adoption_by_type(self, mock_async_client):
        resource = AsyncStats(mock_async_client)
        params = {"type": "military"}
        await resource.adoption_by_type(params)
        mock_async_client.get.assert_called_once_with(
            "/v1/stats/adoption/by-type", query=params
        )

    async def test_action_types(self, mock_async_client):
        resource = AsyncStats(mock_async_client)
        params = {"category": "pistol"}
        await resource.action_types(params)
        mock_async_client.get.assert_called_once_with(
            "/v1/stats/action-types", query=params
        )

    async def test_feature_frequency(self, mock_async_client):
        resource = AsyncStats(mock_async_client)
        params = {"min_count": 5}
        await resource.feature_frequency(params)
        mock_async_client.get.assert_called_once_with(
            "/v1/stats/feature-frequency", query=params
        )

    async def test_caliber_popularity_by_era(self, mock_async_client):
        resource = AsyncStats(mock_async_client)
        params = {"era": "cold-war"}
        await resource.caliber_popularity_by_era(params)
        mock_async_client.get.assert_called_once_with(
            "/v1/stats/caliber-popularity-by-era", query=params
        )



from ._ammunition import Ammunition, AsyncAmmunition
from ._calibers import AsyncCalibers, Calibers
from ._categories import AsyncCategories, Categories
from ._conflicts import AsyncConflicts, Conflicts
from ._countries import AsyncCountries, Countries
from ._data_quality import AsyncDataQuality, DataQuality
from ._favorites import AsyncFavorites, Favorites
from ._firearms import AsyncFirearms, Firearms
from ._game import AsyncGame, Game
from ._game_stats import AsyncGameStats_, GameStats_
from ._manufacturers import AsyncManufacturers, Manufacturers
from ._reports import AsyncReports, Reports
from ._stats import AsyncStats, Stats
from ._support import AsyncSupport, Support
from ._usage import AsyncUsage, Usage
from ._webhooks import AsyncWebhooks, Webhooks

__all__ = [
    "Ammunition",
    "AsyncAmmunition",
    "AsyncCalibers",
    "AsyncCategories",
    "AsyncConflicts",
    "AsyncCountries",
    "AsyncDataQuality",
    "AsyncFavorites",
    "AsyncFirearms",
    "AsyncGame",
    "AsyncGameStats_",
    "AsyncManufacturers",
    "AsyncReports",
    "AsyncStats",
    "AsyncSupport",
    "AsyncUsage",
    "AsyncWebhooks",
    "Calibers",
    "Categories",
    "Conflicts",
    "Countries",
    "DataQuality",
    "Favorites",
    "Firearms",
    "Game",
    "GameStats_",
    "Manufacturers",
    "Reports",
    "Stats",
    "Support",
    "Usage",
    "Webhooks",
]



"""Tests for the Categories and AsyncCategories resource classes."""

from __future__ import annotations

from urllib.parse import quote

from gunspec._resources._categories import AsyncCategories, Categories


class TestCategories:
    """Sync Categories resource tests."""

    def test_list(self, mock_sync_client):
        resource = Categories(mock_sync_client)
        resource.list()
        mock_sync_client.get.assert_called_once_with("/v1/categories")

    def test_get_firearms(self, mock_sync_client):
        resource = Categories(mock_sync_client)
        params = {"page": 1, "limit": 10}
        resource.get_firearms("pistols", params)
        mock_sync_client.get_paginated.assert_called_once_with(
            f"/v1/categories/{quote('pistols', safe='')}/firearms", query=params
        )

    def test_get_firearms_no_params(self, mock_sync_client):
        resource = Categories(mock_sync_client)
        resource.get_firearms("rifles")
        mock_sync_client.get_paginated.assert_called_once_with(
            f"/v1/categories/{quote('rifles', safe='')}/firearms", query=None
        )


class TestAsyncCategories:
    """Async Categories resource tests."""

    async def test_list(self, mock_async_client):
        resource = AsyncCategories(mock_async_client)
        await resource.list()
        mock_async_client.get.assert_called_once_with("/v1/categories")

    async def test_get_firearms(self, mock_async_client):
        resource = AsyncCategories(mock_async_client)
        params = {"page": 1, "limit": 10}
        await resource.get_firearms("pistols", params)
        mock_async_client.get_paginated.assert_called_once_with(
            f"/v1/categories/{quote('pistols', safe='')}/firearms", query=params
        )

    async def test_get_firearms_no_params(self, mock_async_client):
        resource = AsyncCategories(mock_async_client)
        await resource.get_firearms("rifles")
        mock_async_client.get_paginated.assert_called_once_with(
            f"/v1/categories/{quote('rifles', safe='')}/firearms", query=None
        )



"""Tests for the Countries and AsyncCountries resource classes."""

from __future__ import annotations

from urllib.parse import quote

from gunspec._resources._countries import AsyncCountries, Countries


class TestCountries:
    """Sync Countries resource tests."""

    def test_list(self, mock_sync_client):
        resource = Countries(mock_sync_client)
        resource.list()
        mock_sync_client.get.assert_called_once_with("/v1/countries")

    def test_get_arsenal(self, mock_sync_client):
        resource = Countries(mock_sync_client)
        resource.get_arsenal("US")
        mock_sync_client.get.assert_called_once_with(
            f"/v1/countries/{quote('US', safe='')}/arsenal"
        )


class TestAsyncCountries:
    """Async Countries resource tests."""

    async def test_list(self, mock_async_client):
        resource = AsyncCountries(mock_async_client)
        await resource.list()
        mock_async_client.get.assert_called_once_with("/v1/countries")

    async def test_get_arsenal(self, mock_async_client):
        resource = AsyncCountries(mock_async_client)
        await resource.get_arsenal("US")
        mock_async_client.get.assert_called_once_with(
            f"/v1/countries/{quote('US', safe='')}/arsenal"
        )



from __future__ import annotations

from typing import TYPE_CHECKING, AsyncIterator, Dict, Iterator
from urllib.parse import quote

if TYPE_CHECKING:
    from .._core._http_client import APIResponse, AsyncHttpClient, PaginatedResponse, SyncHttpClient


class Ammunition:
    """Sync ammunition resource."""

    def __init__(self, client: SyncHttpClient) -> None:
        self._client = client

    def list(self, params: Dict | None = None) -> PaginatedResponse:
        return self._client.get_paginated("/v1/ammunition", query=params)

    def list_auto_paging(self, params: Dict | None = None) -> Iterator:
        page = (params or {}).get("page", 1)
        while True:
            result = self.list({**(params or {}), "page": page})
            yield from result.data
            if not result.pagination.total_pages or page >= result.pagination.total_pages:
                break
            page += 1

    def get(self, id: str) -> APIResponse:
        return self._client.get(f"/v1/ammunition/{quote(id, safe='')}")

    def get_bullet_svg(self, id: str) -> str:
        response = self._client.get(f"/v1/ammunition/{quote(id, safe='')}/bullet.svg")
        return response.data

    def ballistics(self, id: str, params: Dict | None = None) -> APIResponse:
        return self._client.get(
            f"/v1/ammunition/{quote(id, safe='')}/ballistics", query=params
        )


class AsyncAmmunition:
    """Async ammunition resource."""

    def __init__(self, client: AsyncHttpClient) -> None:
        self._client = client

    async def list(self, params: Dict | None = None) -> PaginatedResponse:
        return await self._client.get_paginated("/v1/ammunition", query=params)

    async def list_auto_paging(self, params: Dict | None = None) -> AsyncIterator:
        page = (params or {}).get("page", 1)
        while True:
            result = await self.list({**(params or {}), "page": page})
            for item in result.data:
                yield item
            if not result.pagination.total_pages or page >= result.pagination.total_pages:
                break
            page += 1

    async def get(self, id: str) -> APIResponse:
        return await self._client.get(f"/v1/ammunition/{quote(id, safe='')}")

    async def get_bullet_svg(self, id: str) -> str:
        response = await self._client.get(f"/v1/ammunition/{quote(id, safe='')}/bullet.svg")
        return response.data

    async def ballistics(self, id: str, params: Dict | None = None) -> APIResponse:
        return await self._client.get(
            f"/v1/ammunition/{quote(id, safe='')}/ballistics", query=params
        )



from __future__ import annotations

from typing import TYPE_CHECKING, AsyncIterator, Dict, Iterator
from urllib.parse import quote

if TYPE_CHECKING:
    from .._core._http_client import APIResponse, AsyncHttpClient, PaginatedResponse, SyncHttpClient


class Firearms:
    """Sync firearms resource."""

    def __init__(self, client: SyncHttpClient) -> None:
        self._client = client

    # -- Collection endpoints --------------------------------------------------

    def list(self, params: Dict | None = None) -> PaginatedResponse:
        return self._client.get_paginated("/v1/firearms", query=params)

    def list_auto_paging(self, params: Dict | None = None) -> Iterator:
        page = (params or {}).get("page", 1)
        while True:
            result = self.list({**(params or {}), "page": page})
            yield from result.data
            if not result.pagination.total_pages or page >= result.pagination.total_pages:
                break
            page += 1

    def search(self, params: Dict) -> PaginatedResponse:
        return self._client.get_paginated("/v1/firearms/search", query=params)

    def compare(self, params: Dict) -> APIResponse:
        return self._client.get("/v1/firearms/compare", query=params)

    def game_meta(self, params: Dict | None = None) -> APIResponse:
        return self._client.get("/v1/firearms/game-meta", query=params)

    def action_types(self) -> APIResponse:
        return self._client.get("/v1/firearms/action-types")

    def filter_options(self) -> APIResponse:
        return self._client.get("/v1/firearms/filter-options")

    def random(self, params: Dict | None = None) -> APIResponse:
        return self._client.get("/v1/firearms/random", query=params)

    def top(self, params: Dict) -> APIResponse:
        return self._client.get("/v1/firearms/top", query=params)

    def head_to_head(self, params: Dict) -> APIResponse:
        return self._client.get("/v1/firearms/head-to-head", query=params)

    def by_feature(self, params: Dict) -> PaginatedResponse:
        return self._client.get_paginated("/v1/firearms/by-feature", query=params)

    def by_action(self, params: Dict) -> PaginatedResponse:
        return self._client.get_paginated("/v1/firearms/by-action", query=params)

    def by_material(self, params: Dict) -> PaginatedResponse:
        return self._client.get_paginated("/v1/firearms/by-material", query=params)

    def by_designer(self, params: Dict) -> PaginatedResponse:
        return self._client.get_paginated("/v1/firearms/by-designer", query=params)

    def power_rating(self, params: Dict | None = None) -> PaginatedResponse:
        return self._client.get_paginated("/v1/firearms/power-rating", query=params)

    def timeline(self, params: Dict | None = None) -> PaginatedResponse:
        return self._client.get_paginated("/v1/firearms/timeline", query=params)

    def by_conflict(self, params: Dict) -> PaginatedResponse:
        return self._client.get_paginated("/v1/firearms/by-conflict", query=params)

    # -- Single-resource endpoints ---------------------------------------------

    def get(self, id: str) -> APIResponse:
        return self._client.get(f"/v1/firearms/{quote(id, safe='')}")

    def get_variants(self, id: str) -> APIResponse:
        return self._client.get(f"/v1/firearms/{quote(id, safe='')}/variants")

    def get_images(self, id: str) -> APIResponse:
        return self._client.get(f"/v1/firearms/{quote(id, safe='')}/images")

    def get_game_stats(self, id: str) -> APIResponse:
        return self._client.get(f"/v1/firearms/{quote(id, safe='')}/game-stats")

    def get_dimensions(self, id: str) -> APIResponse:
        return self._client.get(f"/v1/firearms/{quote(id, safe='')}/dimensions")

    def get_users(self, id: str) -> APIResponse:
        return self._client.get(f"/v1/firearms/{quote(id, safe='')}/users")

    def get_family_tree(self, id: str) -> APIResponse:
        return self._client.get(f"/v1/firearms/{quote(id, safe='')}/family-tree")

    def get_similar(self, id: str) -> APIResponse:
        return self._client.get(f"/v1/firearms/{quote(id, safe='')}/similar")

    def get_adoption_map(self, id: str) -> APIResponse:
        return self._client.get(f"/v1/firearms/{quote(id, safe='')}/adoption-map")

    def get_game_profile(self, id: str) -> APIResponse:
        return self._client.get(f"/v1/firearms/{quote(id, safe='')}/game-profile")

    def get_silhouette(self, id: str, params: Dict | None = None) -> APIResponse:
        return self._client.get(f"/v1/firearms/{quote(id, safe='')}/silhouette", query=params)

    def calculate(self, id: str, params: Dict) -> APIResponse:
        return self._client.get(f"/v1/firearms/{quote(id, safe='')}/calculate", query=params)

    def load(self, id: str, params: Dict | None = None) -> APIResponse:
        return self._client.get(f"/v1/firearms/{quote(id, safe='')}/load", query=params)


class AsyncFirearms:
    """Async firearms resource."""

    def __init__(self, client: AsyncHttpClient) -> None:
        self._client = client

    # -- Collection endpoints --------------------------------------------------

    async def list(self, params: Dict | None = None) -> PaginatedResponse:
        return await self._client.get_paginated("/v1/firearms", query=params)

    async def list_auto_paging(self, params: Dict | None = None) -> AsyncIterator:
        page = (params or {}).get("page", 1)
        while True:
            result = await self.list({**(params or {}), "page": page})
            for item in result.data:
                yield item
            if not result.pagination.total_pages or page >= result.pagination.total_pages:
                break
            page += 1

    async def search(self, params: Dict) -> PaginatedResponse:
        return await self._client.get_paginated("/v1/firearms/search", query=params)

    async def compare(self, params: Dict) -> APIResponse:
        return await self._client.get("/v1/firearms/compare", query=params)

    async def game_meta(self, params: Dict | None = None) -> APIResponse:
        return await self._client.get("/v1/firearms/game-meta", query=params)

    async def action_types(self) -> APIResponse:
        return await self._client.get("/v1/firearms/action-types")

    async def filter_options(self) -> APIResponse:
        return await self._client.get("/v1/firearms/filter-options")

    async def random(self, params: Dict | None = None) -> APIResponse:
        return await self._client.get("/v1/firearms/random", query=params)

    async def top(self, params: Dict) -> APIResponse:
        return await self._client.get("/v1/firearms/top", query=params)

    async def head_to_head(self, params: Dict) -> APIResponse:
        return await self._client.get("/v1/firearms/head-to-head", query=params)

    async def by_feature(self, params: Dict) -> PaginatedResponse:
        return await self._client.get_paginated("/v1/firearms/by-feature", query=params)

    async def by_action(self, params: Dict) -> PaginatedResponse:
        return await self._client.get_paginated("/v1/firearms/by-action", query=params)

    async def by_material(self, params: Dict) -> PaginatedResponse:
        return await self._client.get_paginated("/v1/firearms/by-material", query=params)

    async def by_designer(self, params: Dict) -> PaginatedResponse:
        return await self._client.get_paginated("/v1/firearms/by-designer", query=params)

    async def power_rating(self, params: Dict | None = None) -> PaginatedResponse:
        return await self._client.get_paginated("/v1/firearms/power-rating", query=params)

    async def timeline(self, params: Dict | None = None) -> PaginatedResponse:
        return await self._client.get_paginated("/v1/firearms/timeline", query=params)

    async def by_conflict(self, params: Dict) -> PaginatedResponse:
        return await self._client.get_paginated("/v1/firearms/by-conflict", query=params)

    # -- Single-resource endpoints ---------------------------------------------

    async def get(self, id: str) -> APIResponse:
        return await self._client.get(f"/v1/firearms/{quote(id, safe='')}")

    async def get_variants(self, id: str) -> APIResponse:
        return await self._client.get(f"/v1/firearms/{quote(id, safe='')}/variants")

    async def get_images(self, id: str) -> APIResponse:
        return await self._client.get(f"/v1/firearms/{quote(id, safe='')}/images")

    async def get_game_stats(self, id: str) -> APIResponse:
        return await self._client.get(f"/v1/firearms/{quote(id, safe='')}/game-stats")

    async def get_dimensions(self, id: str) -> APIResponse:
        return await self._client.get(f"/v1/firearms/{quote(id, safe='')}/dimensions")

    async def get_users(self, id: str) -> APIResponse:
        return await self._client.get(f"/v1/firearms/{quote(id, safe='')}/users")

    async def get_family_tree(self, id: str) -> APIResponse:
        return await self._client.get(f"/v1/firearms/{quote(id, safe='')}/family-tree")

    async def get_similar(self, id: str) -> APIResponse:
        return await self._client.get(f"/v1/firearms/{quote(id, safe='')}/similar")

    async def get_adoption_map(self, id: str) -> APIResponse:
        return await self._client.get(f"/v1/firearms/{quote(id, safe='')}/adoption-map")

    async def get_game_profile(self, id: str) -> APIResponse:
        return await self._client.get(f"/v1/firearms/{quote(id, safe='')}/game-profile")

    async def get_silhouette(self, id: str, params: Dict | None = None) -> APIResponse:
        return await self._client.get(
            f"/v1/firearms/{quote(id, safe='')}/silhouette", query=params
        )

    async def calculate(self, id: str, params: Dict) -> APIResponse:
        return await self._client.get(
            f"/v1/firearms/{quote(id, safe='')}/calculate", query=params
        )

    async def load(self, id: str, params: Dict | None = None) -> APIResponse:
        return await self._client.get(
            f"/v1/firearms/{quote(id, safe='')}/load", query=params
        )



from __future__ import annotations

from typing import TYPE_CHECKING
from urllib.parse import quote

if TYPE_CHECKING:
    from .._core._http_client import APIResponse, AsyncHttpClient, SyncHttpClient


class Countries:
    """Sync countries resource."""

    def __init__(self, client: SyncHttpClient) -> None:
        self._client = client

    def list(self) -> APIResponse:
        return self._client.get("/v1/countries")

    def get_arsenal(self, code: str) -> APIResponse:
        return self._client.get(f"/v1/countries/{quote(code, safe='')}/arsenal")


class AsyncCountries:
    """Async countries resource."""

    def __init__(self, client: AsyncHttpClient) -> None:
        self._client = client

    async def list(self) -> APIResponse:
        return await self._client.get("/v1/countries")

    async def get_arsenal(self, code: str) -> APIResponse:
        return await self._client.get(f"/v1/countries/{quote(code, safe='')}/arsenal")



from __future__ import annotations

import os
from typing import Dict, Optional


def resolve_api_key(api_key: Optional[str] = None) -> Optional[str]:
    """Resolve the API key from the explicit parameter or GUNSPEC_API_KEY env var.

    Resolution order:
    1. Explicit ``api_key`` parameter.
    2. ``GUNSPEC_API_KEY`` environment variable.
    3. ``None`` (anonymous / unauthenticated).
    """
    if api_key is not None and api_key != "":
        return api_key
    return os.environ.get("GUNSPEC_API_KEY") or None


def build_auth_headers(api_key: Optional[str] = None) -> Dict[str, str]:
    """Build the authentication headers for a request.

    Returns ``{"X-API-Key": key}`` when a key is available, or an empty dict
    for anonymous mode.
    """
    if api_key is not None and api_key != "":
        return {"X-API-Key": api_key}
    return {}



"""Tests for the DataQuality and AsyncDataQuality resource classes."""

from __future__ import annotations

from gunspec._resources._data_quality import AsyncDataQuality, DataQuality


class TestDataQuality:
    """Sync DataQuality resource tests."""

    def test_coverage(self, mock_sync_client):
        resource = DataQuality(mock_sync_client)
        resource.coverage()
        mock_sync_client.get.assert_called_once_with("/v1/data/coverage")

    def test_confidence(self, mock_sync_client):
        resource = DataQuality(mock_sync_client)
        params = {"min_score": 0.8}
        resource.confidence(params)
        mock_sync_client.get_paginated.assert_called_once_with(
            "/v1/data/confidence", query=params
        )

    def test_confidence_no_params(self, mock_sync_client):
        resource = DataQuality(mock_sync_client)
        resource.confidence()
        mock_sync_client.get_paginated.assert_called_once_with(
            "/v1/data/confidence", query=None
        )


class TestAsyncDataQuality:
    """Async DataQuality resource tests."""

    async def test_coverage(self, mock_async_client):
        resource = AsyncDataQuality(mock_async_client)
        await resource.coverage()
        mock_async_client.get.assert_called_once_with("/v1/data/coverage")

    async def test_confidence(self, mock_async_client):
        resource = AsyncDataQuality(mock_async_client)
        params = {"min_score": 0.8}
        await resource.confidence(params)
        mock_async_client.get_paginated.assert_called_once_with(
            "/v1/data/confidence", query=params
        )

    async def test_confidence_no_params(self, mock_async_client):
        resource = AsyncDataQuality(mock_async_client)
        await resource.confidence()
        mock_async_client.get_paginated.assert_called_once_with(
            "/v1/data/confidence", query=None
        )



from __future__ import annotations

from typing import TYPE_CHECKING, Dict
from urllib.parse import quote

if TYPE_CHECKING:
    from .._core._http_client import APIResponse, AsyncHttpClient, PaginatedResponse, SyncHttpClient


class Webhooks:
    """Sync webhooks resource."""

    def __init__(self, client: SyncHttpClient) -> None:
        self._client = client

    def list(self, params: Dict | None = None) -> PaginatedResponse:
        return self._client.get_paginated("/v1/me/webhooks", query=params)

    def create(self, params: Dict) -> APIResponse:
        return self._client.post("/v1/me/webhooks", body=params)

    def get(self, id: str) -> APIResponse:
        return self._client.get(f"/v1/me/webhooks/{quote(id, safe='')}")

    def update(self, id: str, params: Dict) -> APIResponse:
        return self._client.put(f"/v1/me/webhooks/{quote(id, safe='')}", body=params)

    def delete(self, id: str) -> APIResponse:
        return self._client.delete(f"/v1/me/webhooks/{quote(id, safe='')}")

    def test(self, id: str) -> APIResponse:
        return self._client.post(f"/v1/me/webhooks/{quote(id, safe='')}/test")


class AsyncWebhooks:
    """Async webhooks resource."""

    def __init__(self, client: AsyncHttpClient) -> None:
        self._client = client

    async def list(self, params: Dict | None = None) -> PaginatedResponse:
        return await self._client.get_paginated("/v1/me/webhooks", query=params)

    async def create(self, params: Dict) -> APIResponse:
        return await self._client.post("/v1/me/webhooks", body=params)

    async def get(self, id: str) -> APIResponse:
        return await self._client.get(f"/v1/me/webhooks/{quote(id, safe='')}")

    async def update(self, id: str, params: Dict) -> APIResponse:
        return await self._client.put(
            f"/v1/me/webhooks/{quote(id, safe='')}", body=params
        )

    async def delete(self, id: str) -> APIResponse:
        return await self._client.delete(f"/v1/me/webhooks/{quote(id, safe='')}")

    async def test(self, id: str) -> APIResponse:
        return await self._client.post(f"/v1/me/webhooks/{quote(id, safe='')}/test")



"""Tests for the Reports and AsyncReports resource classes."""

from __future__ import annotations

from gunspec._resources._reports import AsyncReports, Reports


class TestReports:
    """Sync Reports resource tests."""

    def test_create(self, mock_sync_client):
        resource = Reports(mock_sync_client)
        params = {"firearm_id": "glock-g17", "reason": "incorrect-spec"}
        resource.create(params)
        mock_sync_client.post.assert_called_once_with(
            "/v1/me/reports", body=params
        )

    def test_list(self, mock_sync_client):
        resource = Reports(mock_sync_client)
        params = {"page": 1, "limit": 10}
        resource.list(params)
        mock_sync_client.get_paginated.assert_called_once_with(
            "/v1/me/reports", query=params
        )

    def test_list_no_params(self, mock_sync_client):
        resource = Reports(mock_sync_client)
        resource.list()
        mock_sync_client.get_paginated.assert_called_once_with(
            "/v1/me/reports", query=None
        )


class TestAsyncReports:
    """Async Reports resource tests."""

    async def test_create(self, mock_async_client):
        resource = AsyncReports(mock_async_client)
        params = {"firearm_id": "glock-g17", "reason": "incorrect-spec"}
        await resource.create(params)
        mock_async_client.post.assert_called_once_with(
            "/v1/me/reports", body=params
        )

    async def test_list(self, mock_async_client):
        resource = AsyncReports(mock_async_client)
        params = {"page": 1, "limit": 10}
        await resource.list(params)
        mock_async_client.get_paginated.assert_called_once_with(
            "/v1/me/reports", query=params
        )

    async def test_list_no_params(self, mock_async_client):
        resource = AsyncReports(mock_async_client)
        await resource.list()
        mock_async_client.get_paginated.assert_called_once_with(
            "/v1/me/reports", query=None
        )



"""Tests for the Calibers and AsyncCalibers resource classes."""

from __future__ import annotations

from urllib.parse import quote

from gunspec._resources._calibers import AsyncCalibers, Calibers


class TestCalibers:
    """Sync Calibers resource tests."""

    def test_list(self, mock_sync_client):
        resource = Calibers(mock_sync_client)
        params = {"page": 1, "limit": 10}
        resource.list(params)
        mock_sync_client.get_paginated.assert_called_once_with(
            "/v1/calibers", query=params
        )

    def test_list_no_params(self, mock_sync_client):
        resource = Calibers(mock_sync_client)
        resource.list()
        mock_sync_client.get_paginated.assert_called_once_with(
            "/v1/calibers", query=None
        )

    def test_compare(self, mock_sync_client):
        resource = Calibers(mock_sync_client)
        params = {"ids": "9mm,45acp"}
        resource.compare(params)
        mock_sync_client.get.assert_called_once_with(
            "/v1/calibers/compare", query=params
        )

    def test_ballistics(self, mock_sync_client):
        resource = Calibers(mock_sync_client)
        params = {"caliber_id": "9mm"}
        resource.ballistics(params)
        mock_sync_client.get.assert_called_once_with(
            "/v1/calibers/ballistics", query=params
        )

    def test_get(self, mock_sync_client):
        resource = Calibers(mock_sync_client)
        resource.get("9mm-luger")
        mock_sync_client.get.assert_called_once_with(
            f"/v1/calibers/{quote('9mm-luger', safe='')}"
        )

    def test_get_firearms(self, mock_sync_client):
        resource = Calibers(mock_sync_client)
        params = {"page": 1}
        resource.get_firearms("9mm-luger", params)
        mock_sync_client.get_paginated.assert_called_once_with(
            f"/v1/calibers/{quote('9mm-luger', safe='')}/firearms", query=params
        )

    def test_get_firearms_no_params(self, mock_sync_client):
        resource = Calibers(mock_sync_client)
        resource.get_firearms("9mm-luger")
        mock_sync_client.get_paginated.assert_called_once_with(
            f"/v1/calibers/{quote('9mm-luger', safe='')}/firearms", query=None
        )

    def test_get_parent_chain(self, mock_sync_client):
        resource = Calibers(mock_sync_client)
        resource.get_parent_chain("9mm-luger")
        mock_sync_client.get.assert_called_once_with(
            f"/v1/calibers/{quote('9mm-luger', safe='')}/parent-chain"
        )

    def test_get_family(self, mock_sync_client):
        resource = Calibers(mock_sync_client)
        resource.get_family("9mm-luger")
        mock_sync_client.get.assert_called_once_with(
            f"/v1/calibers/{quote('9mm-luger', safe='')}/family"
        )

    def test_get_ammunition(self, mock_sync_client):
        resource = Calibers(mock_sync_client)
        params = {"page": 1}
        resource.get_ammunition("9mm-luger", params)
        mock_sync_client.get_paginated.assert_called_once_with(
            f"/v1/calibers/{quote('9mm-luger', safe='')}/ammunition", query=params
        )

    def test_get_ammunition_no_params(self, mock_sync_client):
        resource = Calibers(mock_sync_client)
        resource.get_ammunition("9mm-luger")
        mock_sync_client.get_paginated.assert_called_once_with(
            f"/v1/calibers/{quote('9mm-luger', safe='')}/ammunition", query=None
        )


class TestAsyncCalibers:
    """Async Calibers resource tests."""

    async def test_list(self, mock_async_client):
        resource = AsyncCalibers(mock_async_client)
        params = {"page": 1, "limit": 10}
        await resource.list(params)
        mock_async_client.get_paginated.assert_called_once_with(
            "/v1/calibers", query=params
        )

    async def test_list_no_params(self, mock_async_client):
        resource = AsyncCalibers(mock_async_client)
        await resource.list()
        mock_async_client.get_paginated.assert_called_once_with(
            "/v1/calibers", query=None
        )

    async def test_compare(self, mock_async_client):
        resource = AsyncCalibers(mock_async_client)
        params = {"ids": "9mm,45acp"}
        await resource.compare(params)
        mock_async_client.get.assert_called_once_with(
            "/v1/calibers/compare", query=params
        )

    async def test_ballistics(self, mock_async_client):
        resource = AsyncCalibers(mock_async_client)
        params = {"caliber_id": "9mm"}
        await resource.ballistics(params)
        mock_async_client.get.assert_called_once_with(
            "/v1/calibers/ballistics", query=params
        )

    async def test_get(self, mock_async_client):
        resource = AsyncCalibers(mock_async_client)
        await resource.get("9mm-luger")
        mock_async_client.get.assert_called_once_with(
            f"/v1/calibers/{quote('9mm-luger', safe='')}"
        )

    async def test_get_firearms(self, mock_async_client):
        resource = AsyncCalibers(mock_async_client)
        params = {"page": 1}
        await resource.get_firearms("9mm-luger", params)
        mock_async_client.get_paginated.assert_called_once_with(
            f"/v1/calibers/{quote('9mm-luger', safe='')}/firearms", query=params
        )

    async def test_get_firearms_no_params(self, mock_async_client):
        resource = AsyncCalibers(mock_async_client)
        await resource.get_firearms("9mm-luger")
        mock_async_client.get_paginated.assert_called_once_with(
            f"/v1/calibers/{quote('9mm-luger', safe='')}/firearms", query=None
        )

    async def test_get_parent_chain(self, mock_async_client):
        resource = AsyncCalibers(mock_async_client)
        await resource.get_parent_chain("9mm-luger")
        mock_async_client.get.assert_called_once_with(
            f"/v1/calibers/{quote('9mm-luger', safe='')}/parent-chain"
        )

    async def test_get_family(self, mock_async_client):
        resource = AsyncCalibers(mock_async_client)
        await resource.get_family("9mm-luger")
        mock_async_client.get.assert_called_once_with(
            f"/v1/calibers/{quote('9mm-luger', safe='')}/family"
        )

    async def test_get_ammunition(self, mock_async_client):
        resource = AsyncCalibers(mock_async_client)
        params = {"page": 1}
        await resource.get_ammunition("9mm-luger", params)
        mock_async_client.get_paginated.assert_called_once_with(
            f"/v1/calibers/{quote('9mm-luger', safe='')}/ammunition", query=params
        )

    async def test_get_ammunition_no_params(self, mock_async_client):
        resource = AsyncCalibers(mock_async_client)
        await resource.get_ammunition("9mm-luger")
        mock_async_client.get_paginated.assert_called_once_with(
            f"/v1/calibers/{quote('9mm-luger', safe='')}/ammunition", query=None
        )



from __future__ import annotations

from typing import TypedDict


# ---------------------------------------------------------------------------
# Pagination (shared base)
# ---------------------------------------------------------------------------


class PaginationParams(TypedDict, total=False):
    """Common pagination parameters accepted by all list endpoints."""

    page: int
    per_page: int
    sort: str
    order: str  # "asc" | "desc"


# ---------------------------------------------------------------------------
# Firearms
# ---------------------------------------------------------------------------


class ListFirearmsParams(PaginationParams, total=False):
    """Parameters for ``GET /v1/firearms``."""

    manufacturer: str
    caliber: str
    category: str
    action_type: str
    country_of_origin: str
    year_introduced_min: int
    year_introduced_max: int
    weight_min: float
    weight_max: float
    barrel_length_min: float
    status: str
    has_3d_model: str
    fields: str


class _SearchFirearmsRequired(TypedDict):
    q: str


class SearchFirearmsParams(_SearchFirearmsRequired, PaginationParams, total=False):
    """Parameters for ``GET /v1/firearms/search``."""


class _CompareFirearmsRequired(TypedDict):
    ids: str


class CompareFirearmsParams(_CompareFirearmsRequired, total=False):
    """Parameters for ``GET /v1/firearms/compare``."""


class GameMetaParams(TypedDict, total=False):
    """Parameters for ``GET /v1/firearms/:id/game/meta``."""

    archetype: str


class RandomFirearmParams(TypedDict, total=False):
    """Parameters for ``GET /v1/firearms/random``."""

    category: str
    country: str


class _TopFirearmsRequired(TypedDict):
    stat: str


class TopFirearmsParams(_TopFirearmsRequired, total=False):
    """Parameters for ``GET /v1/firearms/top``."""

    category: str
    limit: int


class _HeadToHeadRequired(TypedDict):
    a: str
    b: str


class HeadToHeadParams(_HeadToHeadRequired, total=False):
    """Parameters for ``GET /v1/firearms/head-to-head``."""


class _ByFeatureRequired(TypedDict):
    feature: str


class ByFeatureParams(_ByFeatureRequired, PaginationParams, total=False):
    """Parameters for ``GET /v1/firearms/by-feature``."""

    category: str


class _ByActionRequired(TypedDict):
    action: str


class ByActionParams(_ByActionRequired, PaginationParams, total=False):
    """Parameters for ``GET /v1/firearms/by-action``."""


class _ByMaterialRequired(TypedDict):
    material: str
    component: str


class ByMaterialParams(_ByMaterialRequired, PaginationParams, total=False):
    """Parameters for ``GET /v1/firearms/by-material``."""


class _ByDesignerRequired(TypedDict):
    designer: str


class ByDesignerParams(_ByDesignerRequired, PaginationParams, total=False):
    """Parameters for ``GET /v1/firearms/by-designer``."""


class PowerRatingParams(PaginationParams, total=False):
    """Parameters for ``GET /v1/firearms/power-rating``."""

    category: str


class TimelineParams(TypedDict, total=False):
    """Parameters for ``GET /v1/firearms/timeline``."""

    page: int
    per_page: int
    sort: str
    order: str
    from_: int  # 'from' is a reserved word
    to: int
    category: str


class _ByConflictRequired(TypedDict):
    conflict: str


class ByConflictParams(_ByConflictRequired, PaginationParams, total=False):
    """Parameters for ``GET /v1/firearms/by-conflict``."""


class _CalculateBallisticsRequired(TypedDict):
    ammo_id: str


class CalculateBallisticsParams(_CalculateBallisticsRequired, total=False):
    """Parameters for ``GET /v1/firearms/:id/calculate``."""


class LoadFirearmParams(TypedDict, total=False):
    """Parameters for ``GET /v1/firearms/:id/load``."""

    ammo_id: str


# ---------------------------------------------------------------------------
# Manufacturers
# ---------------------------------------------------------------------------


class ListManufacturersParams(PaginationParams, total=False):
    """Parameters for ``GET /v1/manufacturers``."""

    country: str


# ---------------------------------------------------------------------------
# Calibers
# ---------------------------------------------------------------------------


class ListCalibersParams(PaginationParams, total=False):
    """Parameters for ``GET /v1/calibers``."""

    cartridge_type: str
    primer_type: str


class _CompareCalibersRequired(TypedDict):
    ids: str


class CompareCalibersParams(_CompareCalibersRequired, total=False):
    """Parameters for ``GET /v1/calibers/compare``."""


class CaliberBallisticsParams(TypedDict, total=False):
    """Parameters for ``GET /v1/calibers/:id/ballistics``."""

    distance: int


# ---------------------------------------------------------------------------
# Ammunition
# ---------------------------------------------------------------------------


class ListAmmunitionParams(PaginationParams, total=False):
    """Parameters for ``GET /v1/ammunition``."""

    caliber_id: str
    bullet_type: str
    is_common: int


class AmmunitionBallisticsParams(TypedDict, total=False):
    """Parameters for ``GET /v1/ammunition/:id/ballistics``."""

    barrel_length_mm: int
    distances: str


# ---------------------------------------------------------------------------
# Stats
# ---------------------------------------------------------------------------


class PopularCalibersParams(TypedDict, total=False):
    """Parameters for ``GET /v1/stats/popular-calibers``."""

    limit: int


class ProlificManufacturersParams(TypedDict, total=False):
    """Parameters for ``GET /v1/stats/prolific-manufacturers``."""

    limit: int
    category: str


class _ByEraRequired(TypedDict):
    decade: str


class ByEraParams(_ByEraRequired, total=False):
    """Parameters for ``GET /v1/stats/by-era``."""


class _AdoptionByCountryRequired(TypedDict):
    code: str


class AdoptionByCountryParams(_AdoptionByCountryRequired, total=False):
    """Parameters for ``GET /v1/stats/adoption/country``."""


class _AdoptionByTypeRequired(TypedDict):
    type: str


class AdoptionByTypeParams(_AdoptionByTypeRequired, total=False):
    """Parameters for ``GET /v1/stats/adoption/type``."""


class ActionTypesParams(TypedDict, total=False):
    """Parameters for ``GET /v1/stats/action-types``."""

    category: str


class FeatureFrequencyParams(TypedDict, total=False):
    """Parameters for ``GET /v1/stats/feature-frequency``."""

    category: str
    limit: int


class CaliberPopularityByEraParams(TypedDict, total=False):
    """Parameters for ``GET /v1/stats/caliber-popularity-by-era``."""

    from_decade: str
    to_decade: str


# ---------------------------------------------------------------------------
# Game
# ---------------------------------------------------------------------------


class BalanceReportParams(TypedDict, total=False):
    """Parameters for ``GET /v1/game/balance``."""

    threshold: int


class TierListParams(TypedDict, total=False):
    """Parameters for ``GET /v1/game/tier-list``."""

    category: str
    stat: str


class _MatchupsRequired(TypedDict):
    a: str
    b: str


class MatchupsParams(_MatchupsRequired, total=False):
    """Parameters for ``GET /v1/game/matchups``."""


class _RoleRosterRequired(TypedDict):
    role: str


class RoleRosterParams(_RoleRosterRequired, total=False):
    """Parameters for ``GET /v1/game/role-roster``."""

    count: int


class _StatDistributionRequired(TypedDict):
    stat: str


class StatDistributionParams(_StatDistributionRequired, total=False):
    """Parameters for ``GET /v1/game/stat-distribution``."""


# ---------------------------------------------------------------------------
# Game Stats Snapshots
# ---------------------------------------------------------------------------


class ListSnapshotFirearmsParams(PaginationParams, total=False):
    """Parameters for ``GET /v1/game-stats/:version/firearms``."""


# ---------------------------------------------------------------------------
# Countries
# ---------------------------------------------------------------------------


class _CountryCodeRequired(TypedDict):
    code: str


class CountryCodeParam(_CountryCodeRequired, total=False):
    """Path parameter for ``GET /v1/countries/:code/arsenal``."""


# ---------------------------------------------------------------------------
# Conflicts
# ---------------------------------------------------------------------------


class ConflictNameQuery(TypedDict, total=False):
    """Parameters for ``GET /v1/conflicts``."""

    name: str


# ---------------------------------------------------------------------------
# Data Quality
# ---------------------------------------------------------------------------


class ConfidenceParams(PaginationParams, total=False):
    """Parameters for ``GET /v1/data/confidence``."""

    below: float


# ---------------------------------------------------------------------------
# Silhouette
# ---------------------------------------------------------------------------


class SilhouetteParams(TypedDict, total=False):
    """Parameters for ``GET /v1/firearms/:id/silhouette``."""

    format: str
    stroke_width: float
    stroke_color: str


# ---------------------------------------------------------------------------
# User-Scoped (via /v1/me)
# ---------------------------------------------------------------------------


class ListFavoritesParams(TypedDict, total=False):
    """Parameters for ``GET /v1/me/favorites``."""

    page: int
    limit: int


class _CreateReportRequired(TypedDict):
    firearm_id: str
    section: str
    issue_type: str
    description: str


class CreateReportParams(_CreateReportRequired, total=False):
    """Body for ``POST /v1/me/reports``."""

    references: list[str]
    suggested_value: str


class ListReportsParams(TypedDict, total=False):
    """Parameters for ``GET /v1/me/reports``."""

    page: int
    per_page: int


class _CreateTicketRequired(TypedDict):
    subject: str
    description: str


class CreateTicketParams(_CreateTicketRequired, total=False):
    """Body for ``POST /v1/me/support``."""

    category: str
    priority: str


class ListTicketsParams(TypedDict, total=False):
    """Parameters for ``GET /v1/me/support``."""

    page: int
    per_page: int
    status: str
    sort: str
    order: str


class _CreateReplyRequired(TypedDict):
    message: str


class CreateReplyParams(_CreateReplyRequired, total=False):
    """Body for ``POST /v1/me/support/:ticketId/replies``."""


class _CreateWebhookRequired(TypedDict):
    url: str
    events: list[str]


class CreateWebhookEndpointParams(_CreateWebhookRequired, total=False):
    """Body for ``POST /v1/me/webhooks``."""

    description: str


class UpdateWebhookEndpointParams(TypedDict, total=False):
    """Body for ``PUT /v1/me/webhooks/:id``."""

    url: str
    description: str | None
    events: list[str]
    active: bool


class ListWebhookEndpointsParams(TypedDict, total=False):
    """Parameters for ``GET /v1/me/webhooks``."""

    page: int
    per_page: int


class UsageParams(TypedDict, total=False):
    """Parameters for ``GET /v1/me/usage``."""

    month: str
    days: int



"""Tests for the Game and AsyncGame resource classes."""

from __future__ import annotations

from gunspec._resources._game import AsyncGame, Game


class TestGame:
    """Sync Game resource tests."""

    def test_balance_report(self, mock_sync_client):
        resource = Game(mock_sync_client)
        params = {"category": "pistol"}
        resource.balance_report(params)
        mock_sync_client.get.assert_called_once_with(
            "/v1/game/balance-report", query=params
        )

    def test_balance_report_no_params(self, mock_sync_client):
        resource = Game(mock_sync_client)
        resource.balance_report()
        mock_sync_client.get.assert_called_once_with(
            "/v1/game/balance-report", query=None
        )

    def test_tier_list(self, mock_sync_client):
        resource = Game(mock_sync_client)
        params = {"version": "1.0"}
        resource.tier_list(params)
        mock_sync_client.get.assert_called_once_with(
            "/v1/game/tier-list", query=params
        )

    def test_tier_list_no_params(self, mock_sync_client):
        resource = Game(mock_sync_client)
        resource.tier_list()
        mock_sync_client.get.assert_called_once_with(
            "/v1/game/tier-list", query=None
        )

    def test_matchups(self, mock_sync_client):
        resource = Game(mock_sync_client)
        params = {"firearm_a": "glock-g17", "firearm_b": "sig-p320"}
        resource.matchups(params)
        mock_sync_client.get.assert_called_once_with(
            "/v1/game/matchups", query=params
        )

    def test_role_roster(self, mock_sync_client):
        resource = Game(mock_sync_client)
        params = {"role": "assault"}
        resource.role_roster(params)
        mock_sync_client.get.assert_called_once_with(
            "/v1/game/role-roster", query=params
        )

    def test_stat_distribution(self, mock_sync_client):
        resource = Game(mock_sync_client)
        params = {"stat": "damage"}
        resource.stat_distribution(params)
        mock_sync_client.get.assert_called_once_with(
            "/v1/game/stat-distribution", query=params
        )


class TestAsyncGame:
    """Async Game resource tests."""

    async def test_balance_report(self, mock_async_client):
        resource = AsyncGame(mock_async_client)
        params = {"category": "pistol"}
        await resource.balance_report(params)
        mock_async_client.get.assert_called_once_with(
            "/v1/game/balance-report", query=params
        )

    async def test_balance_report_no_params(self, mock_async_client):
        resource = AsyncGame(mock_async_client)
        await resource.balance_report()
        mock_async_client.get.assert_called_once_with(
            "/v1/game/balance-report", query=None
        )

    async def test_tier_list(self, mock_async_client):
        resource = AsyncGame(mock_async_client)
        params = {"version": "1.0"}
        await resource.tier_list(params)
        mock_async_client.get.assert_called_once_with(
            "/v1/game/tier-list", query=params
        )

    async def test_tier_list_no_params(self, mock_async_client):
        resource = AsyncGame(mock_async_client)
        await resource.tier_list()
        mock_async_client.get.assert_called_once_with(
            "/v1/game/tier-list", query=None
        )

    async def test_matchups(self, mock_async_client):
        resource = AsyncGame(mock_async_client)
        params = {"firearm_a": "glock-g17", "firearm_b": "sig-p320"}
        await resource.matchups(params)
        mock_async_client.get.assert_called_once_with(
            "/v1/game/matchups", query=params
        )

    async def test_role_roster(self, mock_async_client):
        resource = AsyncGame(mock_async_client)
        params = {"role": "assault"}
        await resource.role_roster(params)
        mock_async_client.get.assert_called_once_with(
            "/v1/game/role-roster", query=params
        )

    async def test_stat_distribution(self, mock_async_client):
        resource = AsyncGame(mock_async_client)
        params = {"stat": "damage"}
        await resource.stat_distribution(params)
        mock_async_client.get.assert_called_once_with(
            "/v1/game/stat-distribution", query=params
        )



from __future__ import annotations

from typing import TYPE_CHECKING, AsyncIterator, Dict, Iterator
from urllib.parse import quote

if TYPE_CHECKING:
    from .._core._http_client import APIResponse, AsyncHttpClient, PaginatedResponse, SyncHttpClient


class Manufacturers:
    """Sync manufacturers resource."""

    def __init__(self, client: SyncHttpClient) -> None:
        self._client = client

    def list(self, params: Dict | None = None) -> PaginatedResponse:
        return self._client.get_paginated("/v1/manufacturers", query=params)

    def list_auto_paging(self, params: Dict | None = None) -> Iterator:
        page = (params or {}).get("page", 1)
        while True:
            result = self.list({**(params or {}), "page": page})
            yield from result.data
            if not result.pagination.total_pages or page >= result.pagination.total_pages:
                break
            page += 1

    def get(self, id: str) -> APIResponse:
        return self._client.get(f"/v1/manufacturers/{quote(id, safe='')}")

    def get_firearms(self, id: str, params: Dict | None = None) -> PaginatedResponse:
        return self._client.get_paginated(
            f"/v1/manufacturers/{quote(id, safe='')}/firearms", query=params
        )

    def get_timeline(self, id: str) -> APIResponse:
        return self._client.get(f"/v1/manufacturers/{quote(id, safe='')}/timeline")

    def get_stats(self, id: str) -> APIResponse:
        return self._client.get(f"/v1/manufacturers/{quote(id, safe='')}/stats")


class AsyncManufacturers:
    """Async manufacturers resource."""

    def __init__(self, client: AsyncHttpClient) -> None:
        self._client = client

    async def list(self, params: Dict | None = None) -> PaginatedResponse:
        return await self._client.get_paginated("/v1/manufacturers", query=params)

    async def list_auto_paging(self, params: Dict | None = None) -> AsyncIterator:
        page = (params or {}).get("page", 1)
        while True:
            result = await self.list({**(params or {}), "page": page})
            for item in result.data:
                yield item
            if not result.pagination.total_pages or page >= result.pagination.total_pages:
                break
            page += 1

    async def get(self, id: str) -> APIResponse:
        return await self._client.get(f"/v1/manufacturers/{quote(id, safe='')}")

    async def get_firearms(self, id: str, params: Dict | None = None) -> PaginatedResponse:
        return await self._client.get_paginated(
            f"/v1/manufacturers/{quote(id, safe='')}/firearms", query=params
        )

    async def get_timeline(self, id: str) -> APIResponse:
        return await self._client.get(f"/v1/manufacturers/{quote(id, safe='')}/timeline")

    async def get_stats(self, id: str) -> APIResponse:
        return await self._client.get(f"/v1/manufacturers/{quote(id, safe='')}/stats")



from __future__ import annotations

from typing import TYPE_CHECKING, Dict
from urllib.parse import quote

if TYPE_CHECKING:
    from .._core._http_client import APIResponse, AsyncHttpClient, PaginatedResponse, SyncHttpClient


class Favorites:
    """Sync favorites resource."""

    def __init__(self, client: SyncHttpClient) -> None:
        self._client = client

    def list(self, params: Dict | None = None) -> PaginatedResponse:
        return self._client.get_paginated("/v1/me/favorites", query=params)

    def add(self, firearm_id: str) -> APIResponse:
        return self._client.post(f"/v1/me/favorites/{quote(firearm_id, safe='')}")

    def remove(self, firearm_id: str) -> APIResponse:
        return self._client.delete(f"/v1/me/favorites/{quote(firearm_id, safe='')}")


class AsyncFavorites:
    """Async favorites resource."""

    def __init__(self, client: AsyncHttpClient) -> None:
        self._client = client

    async def list(self, params: Dict | None = None) -> PaginatedResponse:
        return await self._client.get_paginated("/v1/me/favorites", query=params)

    async def add(self, firearm_id: str) -> APIResponse:
        return await self._client.post(f"/v1/me/favorites/{quote(firearm_id, safe='')}")

    async def remove(self, firearm_id: str) -> APIResponse:
        return await self._client.delete(f"/v1/me/favorites/{quote(firearm_id, safe='')}")
